!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Module to compute the correction to the large scale power spectrum. To change output!!!!!
!redshifts, modify the array in the last subroutine!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


module LargeScales

use settings
use perturbations
use background
use correlation

implicit none 

real(mcp),public :: dT21LSmeanTab(Na)
real(mcp),public :: dT21LSrmsTab(Na)

contains


	subroutine compute_dT21LSmeanTab
	integer, parameter :: MAXITER = 15
    real(mcp) ::  prec,obtprec
    real(mcp) :: t(0:MAXITER,0:MAXITER)
    real(mcp) :: pas,r,s,ta	
 
	integer :: niter  ! number of actual iterations
    integer :: itermin  ! minimal number of iterations
	integer :: itermax        ! maximal number of iterations
	
	integer :: i,j,l,int_s
#ifdef MPI	
	real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na)
	
	if (omp%rank .eq. my_mpi_root) write(*, '(50a)') "**********************************************100%" 
	
    CALL init_mpi_params(Na)
	
	do i =1, omp%size
    	recvcounts(i) = recvcounts(i)
        displs(i) = displs(i)
    enddo

	itermin = 1
	itermax = 50
	
	!Romberg integration:
	allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
#else
	itermin = 1
	itermax = 50
	write(*, '(50a)') "**********************************************100%"
#endif

#ifdef MPI
    do l = omp%istart, omp%iend 
#else
	do l = 1, Na
#endif
		if (itermax>MAXITER)  itermax=MAXITER
    
    	r = Prob3D(0.0d0)*T21tot(atab(l),0.0d0)
    	ta = (r + Prob3D(vbcmax)*T21tot(atab(l),vbcmax)) / 2.d0
    	niter=0
    	pas=vbcmax
    	t(0,0)=ta*pas
    
100 	niter=niter+1
    	pas=pas/2.d0
    	s=ta
    
    	do i=1, 2**niter-1
    		s = s + Prob3D(0.0d0+pas*i)*T21tot(atab(l),(0.0d0+pas*i))
    	end do
    
    	t(0,niter)=s*pas
    	r=1.d0
    
    	do i=1, niter
     
    		r=r*4.d0
        	j=niter-i
        	t(i,j)=(r*t(i-1,j+1) - t(i-1,j))/(r-1.d0)
        
    	end do
    
    	obtprec = DABS(t(niter,0) - t(niter-1,0))
    
    	if (niter > itermax)  goto 200
    	if (niter<itermin)  goto 100
    	if (obtprec>prec)   goto 100
#ifdef MPI
200 	temp(int_s) = t(niter,0)
#else   
200		dT21LSmeanTab(l) = t(niter,0)
#endif
		int_s = int_s+1
		if(mod(l,4) .eq. 0) write(*,'(a)',advance='no')  '*'
	enddo
#ifdef MPI	
	call MPI_ALLGATHERV(temp, omp%steps, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
	int_s = 1
    
    do j = 1, Na        
           dT21LSmeanTab(j) = tottemp(int_s)
    	   int_s = int_s + 1 
    	   !if (omp%rank .eq.  my_mpi_root) write(*,*) atab(j), dT21LSmeanTab(j)   	
    enddo 
	
	deallocate(temp)
#endif

	end subroutine compute_dT21LSmeanTab
	
	
	subroutine compute_dT21LS_RMSTab
	
	integer, parameter :: MAXITER = 15
    real(mcp) ::  prec,obtprec
    real(mcp) :: t(0:MAXITER,0:MAXITER)
    real(mcp) :: pas,r,s,ta	
 
	integer :: niter  ! number of actual iterations
    integer :: itermin  ! minimal number of iterations
	integer :: itermax        ! maximal number of iterations
	
	integer :: i,j,l,int_s
#ifdef MPI	
	real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na)
	
	CALL init_mpi_params(Na)
	
	do i =1, omp%size
    	recvcounts(i) = recvcounts(i)
        displs(i) = displs(i)
    enddo

	itermin = 1
	itermax = 50
	
	!Romberg integration:
	allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
#else
	itermin = 1
	itermax = 50
#endif

#ifdef MPI
    do l = omp%istart, omp%iend 
#else
	do l = 1, Na
#endif
		if (itermax>MAXITER)  itermax=MAXITER
    
    	r = Prob3D(0.0d0)*(T21tot(atab(l),0.0d0)-dT21LSmeanTab(l))**2
    	ta = (r + Prob3D(vbcmax)*(T21tot(atab(l),vbcmax)-dT21LSmeanTab(l))**2) / 2.d0
    	niter=0
    	pas=vbcmax
    	t(0,0)=ta*pas
    
100 	niter=niter+1
    	pas=pas/2.d0
    	s=ta
    
    	do i=1, 2**niter-1
    		s = s + Prob3D(0.0d0+pas*i)*(T21tot(atab(l),(0.0d0+pas*i))-dT21LSmeanTab(l))**2
    	end do
    
    	t(0,niter)=s*pas
    	r=1.d0
    
    	do i=1, niter
     
    		r=r*4.d0
        	j=niter-i
        	t(i,j)=(r*t(i-1,j+1) - t(i-1,j))/(r-1.d0)
        
    	end do
    
    	obtprec = DABS(t(niter,0) - t(niter-1,0))
    
    	if (niter > itermax)  goto 200
    	if (niter<itermin)  goto 100
    	if (obtprec>prec)   goto 100
#ifdef MPI
200 	temp(int_s) = t(niter,0)
#else
200		dT21LSrmsTab(l) =  sqrt(t(niter,0))
#endif
		int_s = int_s+1
		if(mod(l,4) .eq. 0) write(*,'(a)',advance='no')  '*'
		
	enddo	
#ifdef MPI	
	call MPI_ALLGATHERV(temp, omp%steps, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
	int_s = 1
    if (omp%rank .eq.  my_mpi_root) write(*,*)
    do j = 1, Na        
           dT21LSrmsTab(j) = sqrt(tottemp(int_s))
    	   int_s = int_s + 1 
    	   !If you want to print out data for Fig. 13 in Al-Hamoud et al 2013, uncomment:
    	   if (omp%rank .eq.  my_mpi_root) write(*,*) atab(j), dT21LSrmsTab(j)   	
    enddo 	
	
	deallocate(temp)
#endif	
	
	end subroutine compute_dT21LS_RMSTab

	!le moment supreme!
	
	
	subroutine compute_LSPS
	
	!if (omp%rank .eq.  my_mpi_root) write(*,*,advance='no') "Which redshift (multiple possible)?"
	
    integer ierror
	integer ::  error,j, i
	character(4)  :: alt1 = '.txt', alt2 = '_', alt3 = '/',alt4 = '.dat'
	character(80) :: output_dir, file_name, output_name
	character(100) :: ps
	real(mcp) :: Del2
	!various redshifts:
	real(mcp) :: zs(4)
	real(mcp) :: dTb21(4)
	real(mcp) :: x,y
	real(mcp) :: y2(Nx)
	real(mcp) :: yp1, ypn
	
	!natural spline
	yp1 = 1.1e+30
	ypn = 1.1e+30
	

	
#ifdef MPI	
	if (omp%rank .eq.  my_mpi_root) then
#endif
	write(*,'(a22)',advance='no') "Name for output file? "
	read(*,*) output_name
	
	output_dir = "/scratch/p/pen/meerburg/21cm/"
	!output_dir = "output/"
	file_name = "deltapower"
	
	!chage these if you want different redshifts
	
	zs(1) = 120.d0 
	zs(2) = 80.d0
	zs(3) = 60.d0
	zs(4) = 30.d0
	
	!compute prefactor
	do i = 1, 4
		x = 1.d0/(1.d0+zs(i))
		call spline(atab,dT21LSrmsTab,Na,yp1,ypn,y2)
		call splint(atab,dT21LSrmsTab,y2,Na,x,y)
		dTb21(i) = y**2 
	enddo 
	
	ps = trim(output_dir)//trim(file_name)//trim(alt2)//trim(output_name)//trim(alt1)
	
	open(unit=21,file=ps,form='formatted',status='replace')
	
	do j = 1, Nkl
	    
	    Del2 = kltab(j)**3*Pv2tab(j)/(2.0d0*pi**2)
	    
		write(21,'(5E17.6)') kltab(j), dTb21(1)*Del2, dTb21(2)*Del2, dTb21(3)*Del2, dTb21(4)*Del2
	enddo
	!write(*,*) ""
	!write(*,'(a32,F5.3,a8,F5.3,a7,F5.3,a7,F5.3,a13)') "File written: k (Mpc^-1) Del2(",zs(1), &
	!				") Del2(",zs(2), ") Del2(",zs(3),") Del2(",zs(4),")in Kelvin^2"
	!write(*,*) ""
	close(21)
#ifdef MPI
	endif
#endif
	end subroutine compute_LSPS

end module LargeScales