!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Module to get the transfer functions at recombination fro CAMB sources !!!!!!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


module Transfer_calculator

use mpi_params
use background
use settings
use IniFile
use constants
use CAMB, only : CAMB_GetResults, CAMBParams, CAMB_SetDefParams, AccuracyBoost, CAMB_GetAge, &
    CAMBParams_Set, MT, CAMBdata, NonLinear_Pk, Nonlinear_lens, Reionization_GetOptDepth, &
    CAMB_GetTransfers,CAMB_FreeCAMBdata,CAMB_InitCAMBdata, CAMB_TransfersToPowers, Transfer_SetForNonlinearLensing, &
    initial_adiabatic,initial_vector,initial_iso_baryon,initial_iso_CDM, initial_iso_neutrino, initial_iso_neutrino_vel, &
    Recombination_Name, reionization_name, power_name, threadnum, version, max_nu, &
    num_redshiftwindows, outNone,transfer_kh,transfer_max, initialpower_readparams, &
    recombination_readparams, reionization_readparams, line_reionization,w_lam,cs2_lam

implicit none 

real(mcp), allocatable :: transferz1000(:,:)
real(mcp), allocatable :: transferz1020(:,:)
integer :: Ntrans

public transferz1000, transferz1020, Ntrans

contains

	subroutine compute_TFS
		

	implicit none
    Type(CAMBparams) :: CP
    integer :: i,ik
    
    call CAMB_SetDefParams(CP)
    w_lam = -1.d0   
    cs2_lam = 1.d0
    call CAMBparams_to_CMBparams(CP)
    
    
#ifdef MPI     
	if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
    if (omp%rank .eq. 0) write (*,'("Age of universe/GYr  = ",f7.3)') CAMB_GetAge(CP)
    if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
#else    
    write(*,'(A39)') "                                       "
    write (*,'("Age of universe/GYr  = ",f7.3)') CAMB_GetAge(CP)
    write(*,'(A39)') "                                       "
#endif

    CP%transfer%redshifts(1)  = 1000.
#ifdef MPI     
	if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
    if (omp%rank .eq. 0) write (*,'("Computing Transfer function at z= ",f7.1)') CP%transfer%redshifts(1)
    if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
#else    
    write(*,'(A39)') "                                       "
    write (*,'("Computing Transfer function at z= ",f7.1)') CP%transfer%redshifts(1)
    write(*,'(A39)') "                                       "
#endif
    call CAMB_GetResults(CP)
    allocate(transferz1000(MT%num_q_trans,3))
	do ik=1,MT%num_q_trans
	 
		if (MT%TransferData(Transfer_kh,ik,1)/=0) then
		 !for testing:
		 !write(*,*) MT%TransferData(1:3,ik,1)
		 transferz1000(ik,1:3) = MT%TransferData(1:3,ik,1)
		 transferz1000(ik,1) = transferz1000(ik,1)*CMBparams%h0
		 !write(*,*) MT%TransferData(Transfer_kh:Transfer_max,ik,1)
		end if
	 end do
	 
	CP%transfer%redshifts(1)  = 1020.
#ifdef MPI     
	if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
    if (omp%rank .eq. 0) write (*,'("Computing Transfer function at z= ",f7.1)') CP%transfer%redshifts(1)
    if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
#else    
    write(*,'(A39)') "                                       "
    write (*,'("Computing Transfer function at z= ",f7.1)') CP%transfer%redshifts(1)
    write(*,'(A39)') "                                       "
#endif	
    call CAMB_GetResults(CP)
    allocate(transferz1020(MT%num_q_trans,3))
	do ik=1,MT%num_q_trans
	 
		!if (MT%TransferData(Transfer_kh,ik,i)/=0) then
		 !for testing:
		 
		 transferz1020(ik,1:3) = MT%TransferData(1:3,ik,1)
		 transferz1020(ik,1) = transferz1020(ik,1)*CMBparams%h0
		 !write(*,*) MT%TransferData(Transfer_kh:Transfer_max,ik,1)
		!end if
	 end do
    Ntrans = MT%num_q_trans

    end subroutine compute_TFS
    	
	subroutine CAMBparams_to_CMBparams(CP)

	Type(CAMBparams) :: CP
	
	
	CP%OnlyTransfers = .True.
	CP%Want_CMB = .false.
	CP%WantTransfer = .true.
	CP%WantScalars = .false.
	CP%WantTensors = .false.
	CP%OutputNormalization=outNone
	CP%H0 = CMBparams%h0*100.d0
	CP%Max_eta_k = 4000.d0	
	CP%omegab = CMBparams%omegab0
	CP%omegac = CMBparams%omegac0
	CP%omegav = 1.d0 - CP%omegac - CP%omegab
	CP%TCMB = CMBparams%Tcmb0
	CP%yhe = CMBparams%YHe
	CP%Num_nu_massless = CMBparams%Neff
	CP%InitPower%ScalarPowerAmp(1) = CMBparams%As
	CP%InitPower%an(1) = CMBparams%ns
	CP%InitPower%k_0_scalar = CMBparams%kpivot
	!CMBparams%fb = CMBparams%omegab0/(CMBparams%omegab0+CMBparams%omegac0)
	
	!setting additional paramaters, copied from the driver.

	!CP%nu_mass_splittings = .true.
    CP%Nu_mass_eigenstates = 0
    if (CP%Nu_mass_eigenstates > max_nu) stop 'too many mass eigenstates'


    CP%Nu_mass_fractions(1)=1  
    if (CP%Nu_mass_eigenstates >1) stop 'must give nu_mass_fractions for the eigenstates'

       
	CP%Transfer%high_precision=  .True.
	CP%transfer%kmax          =  kmax
	CP%transfer%k_per_logint  =  0
	CP%transfer%num_redshifts =  1

	CP%transfer%kmax=CP%transfer%kmax*(CP%h0/100._mcp)

    !reionization:
    CP%Reion%Reionization = .False.
    CP%Reion%use_optical_depth  = .True.
    CP%Reion%optical_depth = 0.089
    !recombination
    !we can use Yacine's code here
    !CP%Recomb%RECFAST_fudge = 1.14
    
	CP%Scalar_initial_condition = initial_adiabatic
	
	CP%AccuratePolarization = .false.
    CP%AccurateReionization = .false.
    CP%AccurateBB = .false.
	
	!for Hyrec (jan 2015 version only)
	CP%Recomb%Pann = RCBparams%sigmav_over_mchi !(in cm^3/s/GeV)
	!for now...
	CP%Recomb%FineSRat = 1.0
	CP%Recomb%EMassRat = 1.0
	
	
	end subroutine CAMBparams_to_CMBparams

end module Transfer_calculator
