!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module setting evolution paramaters and initialziing several tables used to store a, k !!
!and vbc!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module background 

use settings

implicit none 

	!6 tables we need 
	real(mcp) :: logTRtab(ntr)
	real(mcp) :: TMovTRtab(ntm)
	
    real(mcp) :: vbctab(Nvbc)
    !small scales
    Real(mcp) :: ktab(Nk)
    !large scales:
    Real(mcp) :: kltab(Nkl)
    real(mcp) :: atab(Na) 
    real(mcp) :: logatab(Na)
    
    real(mcp) :: xtab(Nx)
    real(mcp) :: Nu0tab(Nu0)
    real(mcp) :: NuXtab(NuX)
    
    Type(cosmopar) :: CMBparams
	Type(derived) :: derCMBparams
	Type(otherp) :: RCBparams
	
	!21 cm stuff
	
	real(mcp), parameter :: L21 = 21.1d0
	real(mcp), parameter :: A10 = 2.85E-15
	real(mcp), parameter :: T10 = 0.068d0
	
	public logTRtab, TMovTRtab, CMBparams, derCMBparams, RCBparams, vbctab, ktab, atab, logatab
	public xtab, Nu0tab, NuXtab, kltab
contains 



subroutine set_cosmology_pars
	!Type(cosmopar) :: CMB
	
	CMBparams%h0 = 0.678
	CMBparams%omegab0 = 0.0456
	CMBparams%omegac0 = 0.227
	CMBparams%Tcmb0 = 2.726 
	CMBparams%YHe = 0.24
	CMBparams%Neff = 3.046
	CMBparams%As = 2.196E-9
	CMBparams%ns = 0.96
	CMBparams%kpivot = 0.05
	CMBparams%fb = CMBparams%omegab0/(CMBparams%omegab0+CMBparams%omegac0)
end subroutine set_cosmology_pars

subroutine set_derived_pars
	
    derCMBparams%omegar0 = 4.4816E-7*((CMBparams%Tcmb0**4)/CMBparams%h0**2)*(1. + 0.227*CMBparams%Neff)
	derCMBparams%omegaL0 = 1.-CMBparams%omegac0-CMBparams%omegab0-derCMBparams%omegar0
	derCMBparams%mHemH = 3.97153
	derCMBparams%fHe = CMBparams%YHe/((1.-CMBparams%YHe)*derCMBparams%mHemH)
	derCMBparams%Hub0 = 100*CMBparams%h0
	derCMBparams%mu  = 1.d0/(CMBparams%YHe/derCMBparams%mHemH  + (1. - CMBparams%YHe))
	derCMBparams%nh0 = 11.224E-6*CMBparams%omegab0*CMBparams%h0**2*(1. - CMBparams%YHe) !in cm^-3 today
	!Decoupling redshift, using the fitting function of Eisenstein & Hu 1998 
	derCMBparams%kmpermpc = 1.d0/(3.086E19)
	derCMBparams%omegamh2  = (CMBparams%omegab0+CMBparams%omegac0)*CMBparams%h0**2
	derCMBparams%bone = 0.313*derCMBparams%omegamh2**(-0.419)*(1. + 0.607*derCMBparams%omegamh2**0.674)
	derCMBparams%btwo = 0.238*derCMBparams%omegamh2**(0.223)
	derCMBparams%zd = 1291.*(1+derCMBparams%bone*derCMBparams%omegamh2**derCMBparams%btwo)  &
	              *(derCMBparams%omegamh2**(0.251)/(1.+0.659*derCMBparams%omegamh2**(0.828)))

end subroutine set_derived_pars

subroutine set_other_pars

	 
	!Boltzmann constant in eV per Kelvin:
	RCBparams%kb = 8.617343E-5
	!Ionization energy in eV
	RCBparams%EI = 13.59828
	!2 s - 1 s 2 - photon decay rate in sec^-1
	RCBparams%L2s = 8.2206
	!Used for detailed balance, with Tr in eV
	RCBparams%SahaCoef = 3.01610E21
	!turn on/off dark matter anihilation 
	RCBparams%DMAN_on = .true.
	!dark matter anihilation 
	RCBparams%sigmav_over_mchi = 1.6E-27 !in cm^3/s/GeV
	RCBparams%HydIoE = 16.6d0 !in Ev
	RCBparams%EvToKg = 1.7826619E-36 !Ev in kg
	
	!go to SI
	RCBparams%sigmav_over_mchiSI = RCBparams%sigmav_over_mchi/1.0E9/RCBparams%EvToKg !cm^3 kg^{-1} s^-1
	RCBparams%HydIoEinKg = RCBparams%HydIoE*RCBparams%EvToKg !in kg
end subroutine set_other_pars

subroutine set_2tabs()

    integer :: j
    
    do j = 1, ntr
    
    	logTRtab(j) = Log(trmin) + (j - 1.d0)*Log(trmax/trmin)/(ntr - 1.d0)
 
    enddo 
    
    do j = 1, ntm
    
        TMovTRtab(j) = tMovtrmin + (j - 1.d0)*(TMovtrmax - TMovtrmin)/(ntm - 1.d0)
    
    enddo 
end subroutine set_2tabs

subroutine set_4tabs()


	integer :: j, i
	real(mcp) :: xtablo(Nxlo)
	real(mcp) :: xtabhi(Nxhi)
	
	do j = 1, Na
	atab(j)  = ainit* exp((j-1)*Log(aend/ainit)/(Na-1))
	!write(*,*) atab(j)-atab(j-1)
	enddo 
	
	do j = 1, Na
	logatab(j) = Log(ainit)+(j-1)*Log(aend/ainit)/(Na-1)
	enddo 
	
	
	do j = 1, Nk
	ktab(j)  = kmin*Exp((j-1)*Log(kmax/kmin)/(Nk-1))
	enddo 
	
	do j = 1, Nkl
	kltab(j)  = klmin*Exp((j-1)*Log(klmax/klmin)/(Nkl-1))
	enddo 
	
	do j = 1, Nvbc
	vbctab(j)  = (j-1)*vbcmax/(Nvbc-1)
	!write(*,*) vbctab(j)
	enddo 
	
	do i = 1, Nxlo
	xtablo(i)  = xmin + (i - 1.d0)*(xint - xmin)/(Nxlo - 1.d0)
	enddo 
	
	do i = 1, Nxhi
    xtabhi(i) = xint*Exp(Log(xmax/xint)/(Nxhi - 1.d0)*(i - 1.d0))
    enddo 
    
    do i = 1, Nxlo
    xtab(i) = xtablo(i)
    enddo 
    
    do  i = Nxlo + 1, Nx 
    xtab(i) = xtabhi(i - Nxlo + 1)
    enddo 
	!have to decide on thse tabs
	do  i = 1, Nu0 
    Nu0tab(i) = (i-1)*1.d0
    enddo 
    
    do  i = 1,  NuX 
    NuXtab(i) = (i-1)*1.d0
    enddo 
end subroutine set_4tabs


end module background