!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module containing definition of relevant background functions !!!!!!!!!!!!!!!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


module bkgfunctions

use settings
use readfiles
use interpolation
use background
use ode_solver
use CAMB, only : recombination_xe, recombination_tm, recombination_ts

implicit none 
	
real(mcp) :: dlnCdlnRTab(1000)
real(mcp) :: dlnAdlnTmTab(Na)
real(mcp), public :: Tgas2tab(Na)
real(mcp), public :: Xe2Bar(Na)


public dlnCdlnRTab, dlnAdlnTmTab

contains 


	real(mcp) function Tcmb(a)
	
		real(mcp) :: a
		
		Tcmb = CMBparams%Tcmb0/a
		
	end function Tcmb
	
	real(mcp) function nhbar(a)
	
		real(mcp) :: a
		
		nhbar = 1.*derCMBparams%nh0/a**3.

    end function nhbar
    
    real function Hubble(a)
    
        real(mcp) :: a
        real(mcp) :: TotDensity
        
        TotDensity = (CMBparams%omegab0+CMBparams%omegac0)*a**(-3) + & 
        				derCMBparams%omegaL0 + derCMBparams%omegar0*a**(-4)
        Hubble = derCMBparams%Hub0*sqrt(TotDensity)
        
    end function Hubble
    
    
    real(mcp) function xebar(a)
    
    real(mcp),intent(in) :: a
    real(mcp) :: ztoa
    real(mcp) :: slope
    integer :: kl, kh, j
    !real(mcp) :: xetab(xezmax,3)
    real(mcp) :: delta_z = -1.0
    real(mcp) :: zinit 
    
    
    !xe from recfast/CAMB?
    !xeCAMB = .false. 
    !which redshift?
    
    if (xeCAMB) then 
    
    xebar = recombination_xe(a)
    
    else
    !xe from precomputed file
    ztoa = 1./a -1.
    zinit = xetab(1,1)
   
    !the data is linear in z, and given by zinit + delta_z*j
    !Interested in j: so j = Int((ztoa-zinit)/delta_z)
    
    kl = floor((ztoa-zinit)/delta_z)+1
    kh = kl+1
    
    slope = (xetab(kl,2)-xetab(kh,2))/(xetab(kl,1)-xetab(kh,1))
    xebar = slope*ztoa + xetab(kl,2)-xetab(kl,1)*slope
    endif
    
    end function xebar
    
    real(mcp) function DxebarDa(a)
    
    real(mcp),intent(in) :: a
    real(mcp) :: eps = 0.000001d0
    
    DxebarDa = (xebar(a+eps)-xebar(a-eps))/(2.0d0*eps)
    
    end function DxebarDa
    
    real(mcp) function Tgasbar(a)
    real(mcp),intent(in) :: a
    real(mcp) :: ztoa
    real(mcp) :: slope
    integer :: kl, kh, j
    real(mcp) :: aTgas, bTgas
    real(mcp) :: delta_z = -1.0
    real(mcp) :: zinit 
    
    if (xeCAMB) then 
    
    	Tgasbar = recombination_tm(a)
    
    else
    
    
    !which redshift?
    ztoa = 1./a -1.
    zinit =xetab(1,1)
   
    !the data is linear in z, and given by zinit + delta_z*j
    !Interested in j: so j = Int((ztoa-zinit)/delta_z)
    
    kl = floor((ztoa-zinit)/delta_z)+1
    kh = kl+1
    
    aTgas = Tcmb(a)*xetab(kl,3)
    bTgas = Tcmb(a)*xetab(kh,3)
	
	 !aTgas = Tgas2tab(kl)
    !bTgas = Tgas2tab(kh)
	
	
	
    slope = (aTgas-bTgas)/(xetab(kl,1)-xetab(kh,1))
    Tgasbar = slope*ztoa + aTgas-xetab(kl,1)*slope    
!    real(mcp) :: a
!    real(mcp) :: yS
!    real(mcp) :: dx, x0
!    
!    x0 = Log(atab(1))   
!    dx = abs(Log(atab(1))-Log(atab(2)))
!
!	call rec_interp1d(x0, dx, Tgas2tab, Na, Log(a), yS) 
	
!	Tgasbar = yS
    endif
    
    end function Tgasbar
    
    real(mcp) function soundspeed2(a)
    
    real(mcp) :: a    
    
    soundspeed2 = 0.008263473/derCMBparams%mu*Tgasbar(a)
    
    end function soundspeed2
    

  
    real(mcp) function Gcompton(a)
    
    real(mcp) :: a    
    real(mcp) :: denom
    
    denom = 1.d0+(1.d0+derCMBparams%fHe)/xebar(a)
    Gcompton = 0.01516667d0*Tcmb(a)**4/denom !in km/s/Mpc
    
    end function Gcompton
    
    
    real(mcp) function Rlymana(a)
    
    real(mcp) :: a    
    real(mcp) :: denom
    denom = (nhbar(a)*(1.d0 - xebar(a))) 
    Rlymana=0.00015109848d0*Hubble(a)/denom  
   
    end function Rlymana
	
	
	real(mcp) function Urate(a)
	!for DM particle anihilation. See e.g. Dvorkin et al 2013
	real(mcp) :: a
	real(mcp) :: chidens
	
	chidens = 3.d0*derCMBparams%kmpermpc**2*derCMBparams%Hub0**2*CMBparams%omegac0/(8.d0*pi*GNewton)/a**3
	!in kg/cm^3
	!note that one could add spatial perturbations to the density
	Urate = RCBparams%sigmav_over_mchiSI*chidens**2/(RCBparams%HydIoEinKg) !in cm^{-3}s^{-1}
	
	end function Urate
	
	
	real(mcp) function Gchi(a)
    !dark matter "anihilation rate". See notes
    !slightly different, because it turns out that in this notation you can't really define a Temp
    real(mcp) :: a    
    real(mcp) :: denom
    real(mcp) :: chidens
    
    chidens = 3.d0*derCMBparams%Hub0**2*CMBparams%omegac0/(8.d0*pi*GNewton)/a**3
    !in kg/cm^3 (km/Mpc)^2
    denom = 9.0d0*(xebar(a)+1.d0+derCMBparams%fHe)*nhbar(a)
    !in cm^{-3}
    
    Gchi = (2.0d0*RCBparams%sigmav_over_mchiSI*chidens**2*derCMBparams%kmpermpc**3/denom/kBoltzmann)* &
    !					(1.d0+2.d0*xebar(a))*(clight*100)**2 !in K*km/s/Mpc
    					(clight*100)**2
    
    end function Gchi

    real(mcp) function Alpha2slog(EnA,EnB)

    real(mcp) :: EnA, EnB
    real(mcp) :: ya(ntr,ntm)
    real(mcp) :: y2aS(ntr,ntm)
    real(mcp) :: yS
    real(mcp) :: dx1, dx2, x10, x20
    logical :: use_spline = .true.
    integer :: i, j
    
    do j = 1, ntm
    	do i = 1, ntr
    	ya(i,j) = aitab(((i - 1)*ntm + j), 1)
    	enddo
    enddo 
    if (use_spline) then 
    	call splie2(logTRtab,TMovTRtab,ya,ntr,ntm,y2aS)
    	call splin2(logTRtab,TMovTRtab,ya,y2aS,ntr,ntm,EnA,EnB,yS)
    else
    	x10 = logTRtab(1)
    	x20 = TMovTRtab(1)
    	dx1 = abs(logTRtab(1)-logTRtab(2))
    	dx2 = abs(TMovTRtab(1)-TMovTRtab(2))
    
    	call rec_interp2d(x10, dx1, x20, dx2, ya, ntr, ntm, EnA, EnB, yS)
    endif
    Alpha2slog = yS	
       
    end function

    
    real(mcp) function Alpha2plog(EnA,EnB)
	
    real(mcp) :: EnA, EnB
    real(mcp) :: ya(ntr,ntm)
    real(mcp) :: y2aP(ntr,ntm)
    real(mcp) :: yP
    real(mcp) :: dx1, dx2, x10, x20
    integer :: i, j
    
    logical :: use_spline = .false.
    
    do j = 1, ntm
    	do i = 1, ntr
    	ya(i,j) = aitab(((i - 1)*ntm + j), 2)
    	enddo
    enddo 
    
    
    if (use_spline) then 
    	call splie2(logTRtab,TMovTRtab,ya,ntr,ntm,y2aP)
    
    	call splin2(logTRtab,TMovTRtab,ya,y2aP,ntr,ntm,EnA,EnB,yP)
    else
    	x10 = logTRtab(1)
    	x20 = TMovTRtab(1)
    	dx1 = abs(logTRtab(1)-logTRtab(2))
    	dx2 = abs(TMovTRtab(1)-TMovTRtab(2))
    
    	call rec_interp2d(x10, dx1, x20, dx2, ya, ntr, ntm, EnA, EnB, yP)
    endif
    
    Alpha2plog = yP	
    end function
 
 
 
    real(mcp) function alphaB(TempR, TempM)
    Real(mcp) :: TempR
    Real(mcp) :: TempM
    real(mcp) :: ratio
    
    ratio  = TempM/TempR
    if (ratio > 1.0) ratio = 0.999999999d0
    alphaB = Alpha2slog(Log(RCBparams%kb*TempR),ratio) + &
                Alpha2plog(Log(RCBparams%kb*TempR),ratio)

   
    end function

    real(mcp) function betaB(TempR)

    Real(mcp) :: TempR
    
    
    betaB = (1.d0/4.d0)*RCBparams%SahaCoef*(RCBparams%kb*TempR)**(3.d0/2.d0)* &
    				exp(-0.25d0*RCBparams%EI/(RCBparams%kb*TempR))*alphaB(TempR, TempR)	
   
    end function
    
 
    real(mcp) function Cpeebles(a)
	
    real(mcp) :: a
    real(mcp) :: denom
    
    
    denom = 1.0d0*betaB(Tcmb(a))+0.75d0*Rlymana(a) + &
              0.25d0*RCBparams%L2s
    CPeebles = 1.0d0-betaB(Tcmb(a))/denom
    
    
    end function Cpeebles

    real(mcp) function dlnCdlnR(a)
    
    real(mcp) :: a
    real(mcp) :: denom
    
    denom = 1.d0 + RCBparams%L2s/(3.d0*Rlymana(a))
    denom = denom*(1.0d0*betaB(Tcmb(a))+0.75d0*Rlymana(a) + &
              0.25d0*RCBparams%L2s)
    dlnCdlnR  = -(betaB(Tcmb(a)))/denom
    
    end function
    
    !for fast call, retabulate and interpolate
    subroutine compute_dlnCdlnRTab
    
    real(mcp) :: a
	integer :: j
    
    do j = 1, 1000 
    	a = 1.0d0/(1.0d0+xetab(j,1))
    	dlnCdlnRTab(j) = dlnCdlnR(a) 
    enddo 
    
    end subroutine compute_dlnCdlnRTab
    
    real(mcp) function fast_dlnCdlnR(a)

    !external compute_dlnCdlnRTab
    real(mcp) :: a
    real(mcp) :: delta_z = -1.0
    real(mcp) :: zinit 
    real(mcp) :: ztoa
    real(mcp) :: slope
    integer :: kl, kh, j
    
        !which redshift?
    ztoa = 1.d0/a -1.d0
    zinit = xetab(1,1)
   
    !the data is linear in z, and given by zinit + delta_z*j
    !Interested in j: so j = Int((ztoa-zinit)/delta_z)
    
    kl = floor((ztoa-zinit)/delta_z)+1
    kh = kl+1


    slope = (dlnCdlnRTab(kl)-dlnCdlnRTab(kh))/(xetab(kl,1)-xetab(kh,1))
    fast_dlnCdlnR = slope*ztoa + dlnCdlnRTab(kl)-xetab(kl,1)*slope  
    
    
    end function fast_dlnCdlnR

    

    subroutine compute_dlnAdlnTmTab
    
    integer :: i
    real(mcp) :: a
    real(mcp) :: T1, T2, T2b
    real(mcp) :: eps = 0.00001d0
    
    do i =1, Na
    
    T1 = Tcmb(atab(i))
    !dont need interpolation. T2b = Tgasbar(atab(i))
!    if(Compute_Tgas) then
!    	T2 = Tgas2tab(i)
!    else
!    	!from file
    	T2 = Tgasbar(atab(i))
!    endif
    !print *, T2,T2b
    dlnAdlnTmTab(i) = T2*(alphaB(T1, T2+eps)-alphaB(T1, T2-eps))/(2.d0*eps)*1.d0/alphaB(T1,T2)
    !print *, dlnAdlnTmTab(i)
    enddo 
   
    end subroutine compute_dlnAdlnTmTab
    
    real(mcp) function dlnAdlnTm(a)

    real(mcp) :: a
    real(mcp) :: yS
    real(mcp) :: dx, x0
    
    x0 = Log(atab(1))   
    dx = abs(Log(atab(1))-Log(atab(2)))

	call rec_interp1d(x0, dx, dlnAdlnTmTab, Na, Log(a), yS) 
	
	dlnAdlnTm = yS
    
    end function dlnAdlnTm
    
    subroutine solve_backgroundT
	
	  implicit none
	!
	  integer, parameter :: neqn = 2
	!
	  real(mcp) :: abserr
	  !external derivs
	  integer :: i_step, j, int_s
	  integer :: iflag
	  integer :: iwork(5)
	  integer :: n_step
	  real(mcp) :: relerr
	  real(mcp) :: t
	  real(mcp) :: t_out
	  real(mcp) :: t_start
	  real(mcp) :: t_stop
	  real(mcp) :: work(6*neqn+3)
	  real(mcp) :: y(neqn)
      
      abserr = 0.000000000000001E+00
	  relerr = 0.00000000001E+00
	
	  iflag = 1
	
	  t_start = ainit
	  t_stop = aend
	
	  t_out = ainit
	  
	  
      !initial condition? Not sure about this
	  y(1) = Tcmb(ainit)
	  y(2) = 1.0d0
	  do i_step = 1, Na
		if (i_step .eq. 1) then 
			Tgas2tab(i_step) = y(1)
			Xe2Bar(i_step) = y(2)
		else
		t = atab(i_step-1)
		t_out = atab(i_step)
	
		call rkf45_doublep(BGgasderivs, neqn, y, t, t_out, relerr, abserr, iflag, work, iwork )

		Tgas2tab(i_step) = y(1)
		
		Xe2Bar(i_step) = y(2)
		write(*,*) Xe2Bar(i_step), alphaB(Tcmb(atab(i_step)),Tcmb(atab(i_step)))
		endif
		

	  enddo 

	  	  
	return
	end subroutine solve_backgroundT
	
	subroutine BGgasderivs( a, y, yp )
	!standalone recombination (if you want, slow)
	  implicit none
	
	  real(mcp) :: a
	  real(mcp) :: y(2)
	  real(mcp) :: yp(2)
	  real(mcp) :: Ha
	  real(mcp) :: GComptonLocal, denomA
	  real(mcp) :: RlymanaLocal, denomB
	  real(mcp) :: CPeeblesLocal, denomC
	  real(mcp) :: aB
	  
	  Ha = Hubble(a)
	  
	  denomA = 1.d0+(1.d0+derCMBparams%fHe)/y(2)
      GComptonLocal = 0.01516667d0*Tcmb(a)**4/denomA !in km/s/Mpc
      
      denomB = (nhbar(a)*(1.d0 - y(2))) 
      RlymanaLocal=0.00015109848d0*Hubble(a)/denomB 
      
      denomC = 1.0d0*betaB(Tcmb(a))+0.75d0*RlymanaLocal + &
              0.25d0*RCBparams%L2s
      CPeeblesLocal = 1.0d0-betaB(Tcmb(a))/denomC
      if (a < 1.d0/600.d0) then 
      !estimate for decoupling. This is a very adhoc way, but really this should all be done in 
      !something like recfast or HyRec 
      aB = alphaB(Tcmb(a),Tcmb(a))
      else 
      aB = alphaB(Tcmb(a),y(1))
      endif
	  !for dark matter anihilation:
	  if(RCBparams%DMAN_on) then 
	  yp(1) = (-2.0d0*Ha*y(1)+GComptonLocal*(Tcmb(a)-y(1)))/(a*Ha) + (1.d0+2*y(2))*Gchi(a)/(a*Ha) 
	  yp(2) =  -CPeeblesLocal*aB*nhbar(a)*y(2)**2/(a*Ha)/derCMBparams%kmpermpc+ &
	  			Urate(a)*(1.0d0-y(2))/(3.0d0*nhbar(a))/(a*Ha)/derCMBparams%kmpermpc
	  else
	  yp(1) = (-2.0d0*Ha*y(1)+GComptonLocal*(Tcmb(a)-y(1)))/(a*Ha)
	  yp(2) = -CPeeblesLocal*aB*nhbar(a)*y(2)**2/(a*Ha)/derCMBparams%kmpermpc
	  endif
	  
	  return
	end subroutine BGgasderivs
    
	    
end module bkgfunctions