!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module to compute derivative coefficients numerically (to 1st and 2nd order)!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


module coefficients

use settings
use background
use bkgfunctions
use interpolation

implicit none


real(mcp) :: coeffHtab(Na)
real(mcp) :: coeffHHtab(Na)
real(mcp) :: coeffTtab(Na)
real(mcp) :: coeffTTtab(Na)
real(mcp) :: coeffHTtab(Na)

public coeffHtab, coeffHHtab, coeffTtab, coeffTTtab, coeffHTtab

contains

	real(mcp) function kHH10(Tm)
	
		real(mcp) :: Tm
		!Collision coefficient from Pritchard & Loeb 2012 in cm^3/s
		kHH10 = 3.1E-11*Tm**(0.357d0)*Exp(-32.d0/Tm)
	
	
	end function kHH10
	
	real(mcp) function Ts(a,nh,Tm)
	
		real(mcp) :: a, nh, Tm
	
		Ts = (Tcmb(a) + (nh*kHH10(Tm)*T10)/A10)/(1.d0+nh*kHH10(Tm)*T10/(A10*Tm))
	
	end function Ts
	
	real(mcp) function tau21(a,nh,Tm)
		
		real(mcp) :: a, nh, Tm
		
		tau21 = 3.0d0*T10/(32.0d0*pi*Ts(a,nh,Tm))*L21**3*(1.d0-xebar(a))* &
						nh*A10/(Hubble(a)*derCMBparams%kmpermpc)
		
    end function tau21
    
    real(mcp) function Tb21(a,nh,Tm)
		
		real(mcp) :: a, nh, Tm
		
		Tb21 = a*tau21(a,nh,Tm)*(Ts(a,nh,Tm)-Tcmb(a))
		
	end function Tb21	
	
	subroutine compute_coeff
	
	
	real(mcp) :: eps = 0.01d0
	real(mcp) :: a, h1, h2, Tg
	integer :: j
	
	do j = 1, Na
	a = atab(j)
	h1 = eps*nhbar(a)
	
	if(Compute_Tgas) then
    	h2 = eps*Tgas2tab(j)
    	Tg = Tgas2tab(j)
    else
    	!from file
    	h2 = eps*Tgasbar(a)
    	Tg = Tgasbar(a)
    endif
	
	coeffHtab(j) = (Tb21(a, nhbar(a)+h1,Tg)-Tb21(a, nhbar(a)-h1,Tg))/(2.0d0*eps)
	
	
	coeffHHtab(j) = (Tb21(a, nhbar(a)+h1,Tg)+Tb21(a, nhbar(a)-h1,Tg))/(2.0d0*(eps)**2) - &
						Tb21(a, nhbar(a),Tg)/((eps)**2)
	
	coeffTtab(j) = (Tb21(a, nhbar(a),Tg+h2)-Tb21(a, nhbar(a),Tg-h2))/(2.0d0*eps)

	coeffTTtab(j) = (Tb21(a, nhbar(a),Tg+h2)+Tb21(a, nhbar(a),Tg-h2))/(2.0d0*(eps)**2) - &
						Tb21(a, nhbar(a),Tg)/((eps)**2)
	
	coeffHTtab(j) = (Tb21(a, nhbar(a)+h1,Tg+h2)-Tb21(a, nhbar(a)-h1,Tg+h2))/(2.0d0*eps)/(2.0d0*eps) - &
					 (Tb21(a, nhbar(a)+h1,Tg-h2)-Tb21(a, nhbar(a)-h1,Tg-h2))/(2.0d0*eps)/(2.0d0*eps) 
					 
	enddo 
	
	end subroutine compute_coeff
	
	real(mcp) function coeffH(a)
	
	real(mcp) :: a
    real(mcp) :: yS
    real(mcp) :: dx, x0
    
    x0 = Log(atab(1))   
    dx = abs(Log(atab(1))-Log(atab(2)))

	call rec_interp1d(x0, dx, coeffHtab, Na, Log(a), yS) 
	
	coeffH = yS
	end function coeffH
	
	
	real(mcp) function coeffHH(a)
	
	real(mcp) :: a
    real(mcp) :: yS
    real(mcp) :: dx, x0
    
    x0 = Log(atab(1))   
    dx = abs(Log(atab(1))-Log(atab(2)))

	call rec_interp1d(x0, dx, coeffHHtab, Na, Log(a), yS) 
	
	coeffHH = yS
	
	end function coeffHH

	real(mcp) function coeffT(a)
	
	real(mcp) :: a
    real(mcp) :: yS
    real(mcp) :: dx, x0
    
    x0 = Log(atab(1))   
    dx = abs(Log(atab(1))-Log(atab(2)))

	call rec_interp1d(x0, dx, coeffTtab, Na, Log(a), yS) 
	
	coeffT = yS
	
	end function coeffT

	real(mcp) function coeffTT(a)
	
	real(mcp) :: a
    real(mcp) :: yS
    real(mcp) :: dx, x0
    
    x0 = Log(atab(1))   
    dx = abs(Log(atab(1))-Log(atab(2)))

	call rec_interp1d(x0, dx, coeffTTtab, Na, Log(a), yS) 
	
	coeffTT = yS
	
	end function coeffTT

	real(mcp) function coeffHT(a)
	
	real(mcp) :: a
    real(mcp) :: yS
    real(mcp) :: dx, x0
    
    x0 = Log(atab(1))   
    dx = abs(Log(atab(1))-Log(atab(2)))

	call rec_interp1d(x0, dx, coeffHTtab, Na, Log(a), yS) 
	
	coeffHT = yS
	
	end function coeffHT


end module coefficients