!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module to compute correlation coefficients c_perp and c_par !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


module correlation

use settings
use background
use initial
use bkgfunctions


implicit none

real(mcp) :: cparatab(Nx)
real(mcp) :: cperptab(Nx)
real(mcp) :: Pv2tab(Nkl)

public cparatab, cperptab, Pv2tab


contains

	real(mcp) function funcpara(kx)
	
		real(mcp) :: kx
	
		funcpara = Sin(kx)/kx - 2.d0*(Sin(kx)-kx*Cos(kx))/kx**3
	
	end function funcpara
	
	real(mcp) function funcperp(kx)
	
		real(mcp) :: kx
	
		funcperp = (Sin(kx)-kx*Cos(kx))/kx**3
	
	end function funcperp

	subroutine compute_corCoeff
	
		real(mcp) :: p0, dp0
		integer :: i, l, j, kmaxint, int_s
		real(mcp) :: powerS, dvb_init, dvc_init, Pvbc_k
		real(mcp) ::TempCperp, TempCpara
		
#ifdef MPI		
		real(mcp), allocatable :: cortemp1(:), cortemp2(:)
		
		!might have to improve integration here, I get some wiggles for small k in the
		!P_v^2. There is some discrepency at large x between correlation coef here and 
		!the ones in Mathematica, this can be the only reason for the wiggles I think... 
		
		!note: fixed, need to increase number of steps at low x.. (increased by a factor of 2). 
		
		cparatab(1) = 1.d0
		cperptab(1) = 1.d0
		
		CALL init_mpi_params(Nx)
 
		
		allocate(cortemp1(recvcounts(omp%rank+1)))
        allocate(cortemp2(recvcounts(omp%rank+1)))

		
		if (omp%rank .eq. my_mpi_root) write(*, '(50a)') "**********************************************100%"
		
		int_s = 1
#else
		cparatab(1) = 1.d0
		cperptab(1) = 1.d0
		write(*, '(50a)') "**********************************************100%"
#endif

#ifdef MPI
		do i = omp%istart, omp%iend
#else
		do i = 1, Nx
#endif
		!write(*,*) i
		if (i .eq. 1) then
#ifdef MPI
		cortemp1(1) = 1.d0
		cortemp2(1) = 1.d0
#else
		cparatab(1) = 1.d0
		cperptab(1) = 1.d0
#endif
		int_s = int_s + 1
		else
		TempCpara = 0.0d0
		TempCperp = 0.0d0
	    p0 = 0.001
	    if (i < 300) then 
	    dp0 = 0.005
	    kmaxint = 2000
		elseif (i .ge. 300 .and. i < 450) then
		!increase resultion for high frequencies:
		!in principle one could make this even faster with some adaptive integration. 
		dp0 = 0.001
		kmaxint = 10000
		else
		dp0 = 0.001
		kmaxint = 10000
		endif
		do l = 1, kmaxint
				powerS = sqrt(Scalarpower(p0))
				dvb_init = Hubble(ainit)*powerS*p0**2*(transferz1020_bar(p0)-transferz1000_bar(p0))/(20.d0*ainit)
				dvc_init = Hubble(ainit)*powerS*p0**2*(transferz1020_cdm(p0)-transferz1000_cdm(p0))/(20.d0*ainit)
			
				Pvbc_k= ainit**2*(dvc_init-dvb_init)**2/p0**3
			
				TempCpara = Pvbc_k*funcpara(p0*xtab(i))*dp0 + TempCpara
				TempCperp = Pvbc_k*funcperp(p0*xtab(i))*dp0 + TempCperp
				p0 = p0 + dp0 
		enddo
#ifdef MPI		
		cortemp1(int_s) = TempCpara/sigma1D**2
		cortemp2(int_s) = TempCperp/sigma1D**2
#else
		cparatab(i) = TempCpara/sigma1D**2
		cperptab(i) = TempCperp/sigma1D**2
#endif
		int_s = int_s+1
		if(mod(i,10) .eq. 0) write(*,'(a)',advance='no')  '*'
		
		endif
		enddo 
#ifdef MPI	    
	   call MPI_ALLGATHERV(cortemp1, omp%steps, MPI_REAL8, &
                       cparatab, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
	   call MPI_ALLGATHERV(cortemp2, omp%steps, MPI_REAL8, &
                       cperptab, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
	  if(omp%rank .eq. my_mpi_root) write(*,'(a)',advance='no')  '*' 
 	  
 	  deallocate(cortemp1)
      deallocate(cortemp2)
#else
	   write(*,'(a)',advance='no')  '*' 
#endif
	
	end subroutine compute_corCoeff
	
	real(mcp) function cpara(x)
	real(mcp) :: x, y
	real(mcp) :: y2(Nx)
	real(mcp) :: yp1, ypn
	!natural spline
	yp1 = 1.1e+30
	ypn = 1.1e+30
	
	call spline(xtab,cparatab,Nx,yp1,ypn,y2)
	call splint(xtab,cparatab,y2,Nx,x,y)
	
	cpara = y
	
	end function cpara
	
	real(mcp) function cperp(x)
	real(mcp) :: x, y
	real(mcp) :: y2(Nx)
	real(mcp) :: yp1, ypn
	!natural spline
	yp1 = 1.1e+30
	ypn = 1.1e+30
	
	call spline(xtab,cperptab,Nx,yp1,ypn,y2)
	call splint(xtab,cperptab,y2,Nx,x,y)
	
	cperp = y
	
	end function cperp
	
	
	subroutine compute_Pv2tab
	
	!set to accurate integration:
	logical :: accuracy = .false.
	
	integer, parameter :: MAXITER = 15
    real(mcp) ::  prec,obtprec
    real(mcp) :: t(0:MAXITER,0:MAXITER)
    real(mcp) :: pas,r,s,ta	
    real(mcp) :: xlp
 
	integer :: niter  ! number of actual iterations
    integer :: itermin  ! minimal number of iterations
	integer :: itermax        ! maximal number of iterations
	
	integer :: i,j,l,int_s
	
	real(mcp) :: temp_rint, dx
	real(mcp) :: SBessel0Temp
#ifdef MPI 		
	real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Nkl)
	
    if (omp%rank .eq. my_mpi_root) write(*, '(50a)') "**********************************************100%" 
#else
	write(*, '(50a)') "**********************************************100%" 
#endif	

#ifdef MPI 	
    CALL init_mpi_params(Nkl)
	
	do i =1, omp%size
    	recvcounts(i) = recvcounts(i)
        displs(i) = displs(i)
    enddo
	
	allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
#endif	
    
	itermin = 1
    
#ifdef MPI    
    do l = omp%istart, omp%iend 
#else	
    do l = 1, Nkl
#endif
    !reduce timing/accuracy lowering max iterations:
	if (l > 152) then 
		itermax = 10
	else
		itermax = 10
	endif
	
		if (itermax>MAXITER)  itermax=MAXITER
    
    	r = 0.0d0
    	ta = (r + xmax*(Sin(kltab(l)*xmax)/(kltab(l)))*(cpara(xmax)**2+2.0d0*cperp(xmax)**2)) / 2.d0
    	niter=0
    	pas=xmax
    	t(0,0)=ta*pas
    
100 	niter=niter+1
    	pas=pas/2.d0
    	s=ta
    
    	do i=1, 2**niter-1
    		s = s + (0.0d0+pas*i)*(Sin(kltab(l)*(0.0d0+pas*i))/(kltab(l)))*(cpara(0.0d0+pas*i)**2+2.0d0*cperp(0.0d0+pas*i)**2)
    	end do
    
    	t(0,niter)=s*pas
    	r=1.d0
    
    	do i=1, niter
     
    		r=r*4.d0
        	j=niter-i
        	t(i,j)=(r*t(i-1,j+1) - t(i-1,j))/(r-1.d0)
        
    	end do
    
    	obtprec = DABS(t(niter,0) - t(niter-1,0))
    
    	if (niter > itermax)  goto 200
    	if (niter<itermin)  goto 100
    	if (obtprec>prec)   goto 100
#ifdef MPI 
200 	temp(int_s) = t(niter,0)
#else
200		Pv2tab(l) = 4.0d0*pi*t(niter,0)/3.0d0
#endif		
		if(mod(l,4) .eq. 0) write(*,'(a)',advance='no')  '*'
#ifdef MPI 
		int_s = int_s+1
#endif		
	enddo
	
#ifdef MPI	
	call MPI_ALLGATHERV(temp, omp%steps, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
	int_s = 1
    
    do j = 1, Nkl        
           Pv2tab(j) = 4.0d0*pi*tottemp(int_s)/3.0d0
    	   int_s = int_s + 1 
    enddo 
	
	deallocate(temp)
#endif	
	end subroutine compute_Pv2tab

end module correlation