program run

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Main program to run computation of modulation on large scales from Vbc effect!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

use settings
use readfiles
use background
use bkgfunctions
use initial
use perturbations
use ode_solver
use coefficients
use correlation
use probabilty
use LargeScales
use Transfer_calculator
use CAMB, only: Recombination_xe

implicit none 

	Type(fluctuations) :: fluc
	real(mcp) :: xe_quadratictab(Na, Nvbc, Nk)
	real(mcp) :: xe_sourceintktab(Na, Nvbc)
	real(mcp) :: Tg_quadratictab(Na, Nvbc, Nk)
	real(mcp) :: Tg_sourceintktab(Na, Nvbc)
	real(mcp) :: db2_intktab(Na, Nvbc)


	integer :: j,i
	real(mcp) :: a
	
	real(mcp) :: xtemp, dxtmp
	
	
#ifdef MPI 
	call MPI_INIT(omp%ierror)
    call MPI_COMM_RANK(MPI_COMM_WORLD, omp%rank, omp%ierror)

#endif	
	
#ifdef MPI
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "This code computes the correction to   "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "large scales from vbc effects.         "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Changes to background cosmology should "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "be made in background.f90              "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
#else
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "This code computes the correction to   "
    write(*,'(A39)') "large scales from vbc effects.         "
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Changes to background cosmology should "
    write(*,'(A39)') "be made in background.f90              "
    write(*,'(A39)') "                                       "
#endif	
	
	
	
	!calling relevant cosmological parameters and reading density perturbations 
	!from file. This can be done for testing, however need to uncomment pertyrbations in
	!pertrubations.f90
	
	!call read_files(fluc)
	
	call set_cosmology_pars
	
    call set_derived_pars
    call set_other_pars
    call set_2tabs()
    call set_4tabs()
    !still need to read this file for its z values (used in some places in the code for 
    !good itteration points). Needs to be fixed
    call read_planckxe
    
    call read_alphainf


    
#ifdef MPI
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Computing transfer functions from CAMB "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Change settings in Trasnfer_calculator "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
#else
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Computing transfer functions from CAMB "
    write(*,'(A39)') "Change settings in Trasnfer_calculator "
    write(*,'(A39)') "                                       "
#endif

    !write(*,*) "i am here 1"
    !solving background equations for the gas temperature instead of using CAMB 
    if (Compute_Tgas) then 
    call solve_backgroundT
    endif
!    
#ifdef MPI
    if (omp%rank .eq.  my_mpi_root) then
    write(*,*) Na
    do i = 1, 200
    
    write(*,*) atab(i),Tgasbar(atab(i)), xebar(atab(i))
    
    enddo 
    endif
#endif         
!#ifdef MPI
!    if (omp%rank .eq.  my_mpi_root) then
!    do i = 1, Na
!    
!    write(*,*) atab(i), Gchi(atab(i)), Gcompton(atab(i))
!    
!    enddo 
!    endif
!#endif 
    !write(*,*) "i am here 2"
    
    call compute_TFS
    call compute_coeff

    !write(*,*) "i am here 3"

    

!#ifdef MPI
!    if (omp%rank .eq.  my_mpi_root) then
!    do i = 1, Na
!    
!    write(*,*) atab(i),xebar(atab(i)), Xe2Bar(i)
!    
!    enddo 
!    endif
!#endif 
    
    
    !comment out/in if you want to read in transfer functions instead of recomputing them 
    !You will have to modify readfiles.f90 accordingly. 
    !call read_transfer


#ifdef MPI
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Setting initial conditions + sigma3D   "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
#else
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Setting initial conditions + sigma3D   "
    write(*,'(A39)') "                                       "
#endif
    call set_initial_conditions()
    call compute_sigma3D
    call set_probtabs
#ifdef MPI
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Computing correlation coeff            "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
#else
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Computing correlation coeff ]          "
    write(*,'(A39)') "                                       "
#endif    
    call compute_corCoeff

#ifdef MPI
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Computing  P_v^2: change integration   "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "accuracy in correlation.f90            "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
#else
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Computing  P_v^2: change integration   "
    write(*,'(A39)') "accuracy in correlation.f90            "
    write(*,'(A39)') "                                       "
#endif   
    call compute_Pv2tab
    call det_NxMinNxMax
	call compute_dlnAdlnTmTab
	call compute_dlnCdlnRTab

    a = 1./(100.)
#ifdef MPI    
    if (omp%rank .eq.  my_mpi_root) then
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "                                       "
    write(*,'(A8,X,1E16.7)') "coefH:   ",coeffH(a) 
    write(*,'(A8,X,1E16.7)') "sigma3D: ",sigma3D 
    write(*,'(A8,X,1E16.7)') "at z =   ",1/a-1.d0
    write(*,'(A8,X,1E16.7)') "Xe       ", xebar(a)
    write(*,'(A8,X,2E16.7)') "TCMB:    ", Tcmb(a)
    write(*,'(A8,X,1E16.7)') "Tgas:    ", Tgasbar(a)
    write(*,'(A8,X,1E16.7)') "cs^2:    ", soundspeed2(a)
    write(*,'(A8,X,1E16.7)') "Gc:      ", Gcompton(a)
    write(*,'(A8,X,1E16.7)') "Hubble:  ", Hubble(a)
    write(*,'(A8,X,1E16.7)') "Rlymana: ", Rlymana(a)
    write(*,'(A8,X,1E16.7)') "nhbar:   ", nhbar(a)
    write(*,'(A8,X,1E16.7)') "n0:      ", derCMBparams%nh0
    write(*,'(A8,X,1E16.7)') "Saha     ", RCBparams%SahaCoef
    write(*,'(A8,X,1E16.7)') "Boltzman:", RCBparams%kb
    write(*,'(A8,X,2E16.7)') "alphaB:  ", alphaB(Tcmb(a),Tgasbar(a))
    write(*,'(A8,X,2E16.7)') "betaB:   ", betaB(Tcmb(a))
	write(*,'(A8,X,2E16.7)') "CPeebs:  ", Cpeebles_full(a,0.d0,0.d0)	
	write(*,'(A8,X,2E16.7)') "Alp2slog:", Alpha2slog(Log(RCBparams%kb*Tcmb(a)),0.15d0)
	write(*,'(A8,X,2E16.7)') "Alp2plog:", Alpha2plog(Log(RCBparams%kb*Tcmb(a)),0.15d0)
	write(*,'(A8,X,2E16.7)') "Pann:    ", RCBparams%sigmav_over_mchi
	write(*,'(A39)') "                                       "
    endif  
#else    
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "                                       "
    write(*,'(A8,X,1E16.7)') "coefH:   ",coeffH(a) 
    write(*,'(A8,X,1E16.7)') "sigma3D: ",sigma3D 
    write(*,'(A8,X,1E16.7)') "at z =   ",1/a-1.d0
    write(*,'(A8,X,1E16.7)') "Xe       ", xebar(a)
    write(*,'(A8,X,2E16.7)') "TCMB:    ", Tcmb(a)
    write(*,'(A8,X,1E16.7)') "Tgas:    ", Tgasbar(a)
    write(*,'(A8,X,1E16.7)') "cs^2:    ", soundspeed2(a)
    write(*,'(A8,X,1E16.7)') "Gc:      ", Gcompton(a)
    write(*,'(A8,X,1E16.7)') "Hubble:  ", Hubble(a)
    write(*,'(A8,X,1E16.7)') "Rlymana: ", Rlymana(a)
    write(*,'(A8,X,1E16.7)') "nhbar:   ", nhbar(a)
    write(*,'(A8,X,1E16.7)') "n0:      ", derCMBparams%nh0
    write(*,'(A8,X,1E16.7)') "Saha     ", RCBparams%SahaCoef
    write(*,'(A8,X,1E16.7)') "Boltzman:", RCBparams%kb
    write(*,'(A8,X,2E16.7)') "alphaB:  ", alphaB(Tcmb(a),Tgasbar(a))
    write(*,'(A8,X,2E16.7)') "betaB:   ", betaB(Tcmb(a))
	write(*,'(A8,X,2E16.7)') "CPeebs:  ", Cpeebles_full(a,0.d0,0.d0)	
	write(*,'(A8,X,2E16.7)') "Alp2slog:", Alpha2slog(Log(RCBparams%kb*Tcmb(a)),0.15d0)
	write(*,'(A8,X,2E16.7)') "Alp2plog:", Alpha2plog(Log(RCBparams%kb*Tcmb(a)),0.15d0)
	write(*,'(A8,X,2E16.7)') "Pann:    ", RCBparams%sigmav_over_mchi
	write(*,'(A39)') ""
#endif

#ifdef MPI
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Solving coupled differential equations "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
#else
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Solving coupled differential equations "
    write(*,'(A39)') "                                       "
#endif

    call solve_deltas
    
    !computing perturbations to second order, and integrating
#ifdef MPI    
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Computing perturbations to second order"
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
#else
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Computing perturbations to second order"
    write(*,'(A39)') "                                       "
#endif
	call compute_xe_quadratic(fluc,xe_quadratictab)	
    call compute_Tgas_quadratic(fluc,Tg_quadratictab)
#ifdef MPI    
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "                                       "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') "Performing Integration over k and vbc  "
    if (omp%rank .eq.  my_mpi_root) write(*,'(A39)') " 									     "
#else
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Performing Integration over k and vbc  "
    write(*,'(A39)') "                                       "
#endif
    call kintegral_desource(xe_quadratictab,xe_sourceintktab)
    call vbcint_desource(xe_sourceintktab)
    call kintegral_dTgsource(Tg_quadratictab,Tg_sourceintktab)
    call vbcint_dTgsource(Tg_sourceintktab)
    call kintegral_db2(db2_intktab)
    call vbcint_db2(db2_intktab)
    call av_db2(db2_intktab)

#ifdef MPI     
	if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
    if (omp%rank .eq. 0) write(*,'(A39)') "Solving for dT2 and de2                "
    if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
#else    
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Solving for dT2 and de2                "
    write(*,'(A39)') "                                       "
#endif
    call solve_dedT
    call compute_21IItot
 
#ifdef MPI     
	if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
    if (omp%rank .eq. 0) write(*,'(A39)') "Computing first order bias             "
    if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
#else    
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Computing first order bias             "
    write(*,'(A39)') "                                       "
#endif
    call compute_dT21LSmeanTab
    call compute_dT21LS_RMSTab

#ifdef MPI     
	if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
    if (omp%rank .eq. 0) write(*,'(A39)') "Writing file with delta power spectrum "
    if (omp%rank .eq. 0) write(*,'(A39)') "                                       "
#else    
    write(*,'(A39)') "                                       "
    write(*,'(A39)') "Writing file with delta power spectrum "
    write(*,'(A39)') "                                       "
#endif

	call compute_LSPS

	if (allocated(transferz1000)) deallocate(transferz1000)
	if (allocated(transferz1020)) deallocate(transferz1020)    
    
    
#ifdef MPI
    call MPI_FINALIZE(omp%ierror)		
#endif

end program run