!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module to compute the initial conditions required to solve the coupled ODE!!!!!!!!!!!!!!!
!I precompute these at the same k as found in the ktab, in order to speed up the code!!!!!
!If one would decide to change the number of k, this should be automatically covered.!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


module initial 

use settings
use background
use readfiles
use interpolation
use bkgfunctions
use Transfer_calculator

implicit none 

	Type(InitialC) :: Init
	real(mcp) :: sigma3D, sigma1D
	public Init, sigma3D, sigma1D

contains

real(mcp) function ScalarPower(p0)

	real(mcp) :: p0
	!primrordial power spectrum, can change that here!
	Scalarpower = CMBparams%As*(p0/CMBparams%kpivot)**(CMBparams%ns-1.d0)

end function ScalarPower

subroutine compute_sigma3D
	
		
	real(mcp) :: Pvbc_k
	integer :: l
	real(mcp) :: dvc_init
	real(mcp) :: dvb_init
	real(mcp) :: p0, dp0
	real(mcp) :: s3D2
	real(mcp) :: powerS
	
	p0 = 0.001
	dp0 = 0.001
	s3D2 = 0.0d0
#ifdef MPI 	
	if (omp%rank .eq. my_mpi_root) write(*, '(50a)') "**********************************************100%" 
#else
	write(*, '(50a)') "**********************************************100%"
#endif
	do l = 1, 10000
	powerS = sqrt(ScalarPower(p0))
    dvb_init = Hubble(ainit)*powerS*p0**2*(transferz1020_bar(p0)-transferz1000_bar(p0))/(20.d0*ainit)
	dvc_init = Hubble(ainit)*powerS*p0**2*(transferz1020_cdm(p0)-transferz1000_cdm(p0))/(20.d0*ainit)
	
	Pvbc_k= ainit**2*(dvc_init-dvb_init)**2/p0**3
	
	s3D2 = Pvbc_k*dp0+s3D2
	p0 = p0 + dp0
#ifdef MPI 	
	if(omp%rank .eq. my_mpi_root .and. mod(l,200) .eq. 0) write(*,'(a)',advance='no')  '*'
#else
	if(mod(l,200) .eq. 0)write(*,'(a)',advance='no')  '*'
#endif
	enddo
	!integrating 
	sigma3D = sqrt(s3D2)
	sigma1D = sigma3D/sqrt(3.0d0)
	
	
	
end subroutine compute_sigma3D




subroutine set_initial_conditions
	
	integer :: i
	real(mcp) :: powerS
	real(mcp) :: p0
	
	do i = 1, Nk
		p0 = ktab(i)
		powerS = sqrt(ScalarPower(p0))
		Init%deltab(i) = powerS*p0**2*(transferz1020_bar(p0)+transferz1000_bar(p0))/2.d0
		Init%deltac(i) = powerS*p0**2*(transferz1020_cdm(p0)+transferz1000_cdm(p0))/2.d0
		Init%deltavb(i) = Hubble(ainit)*powerS*p0**2*(transferz1020_bar(p0)-transferz1000_bar(p0))/(20.d0*ainit)
	    Init%deltavc(i) = Hubble(ainit)*powerS*p0**2*(transferz1020_cdm(p0)-transferz1000_cdm(p0))/(20.d0*ainit)
	enddo
		
end subroutine set_initial_conditions


real(mcp) function transferz1000_cdm(p0)
	real(mcp) :: y2tmp(Ntrans)
	real(mcp) :: y
	real(mcp) :: p0
	

	call spline(transferz1000(1:Ntrans,1),transferz1000(1:Ntrans,2), Ntrans, 1.d30,1.d30,y2tmp)
	call splint(transferz1000(1:Ntrans,1),transferz1000(1:Ntrans,2),y2tmp,Ntrans,p0,y)
	
	transferz1000_cdm = y
	
end function transferz1000_cdm

real(mcp) function transferz1000_bar(p0)
	real(mcp) :: y2tmp(Ntrans)
	real(mcp) :: y
	real(mcp) :: p0

	call spline(transferz1000(1:Ntrans,1),transferz1000(1:Ntrans,3), Ntrans, 1.d30,1.d30,y2tmp)
	call splint(transferz1000(1:Ntrans,1),transferz1000(1:Ntrans,3),y2tmp,Ntrans,p0,y)
	
	transferz1000_bar = y
	
end function transferz1000_bar

real(mcp) function transferz1020_cdm(p0)
	real(mcp) :: y2tmp(Ntrans)
	real(mcp) :: y
	real(mcp) :: p0

	call spline(transferz1020(1:Ntrans,1),transferz1020(1:Ntrans,2), Ntrans, 1.d30,1.d30,y2tmp)
	call splint(transferz1020(1:Ntrans,1),transferz1020(1:Ntrans,2),y2tmp,Ntrans,p0,y)
	
	transferz1020_cdm = y
	
end function transferz1020_cdm

real(mcp) function transferz1020_bar(p0)
	real(mcp) :: y2tmp(Ntrans)
	real(mcp) :: y
	real(mcp) :: p0

	call spline(transferz1020(1:Ntrans,1),transferz1020(1:Ntrans,3), Ntrans, 1.d30,1.d30,y2tmp)
	call splint(transferz1020(1:Ntrans,1),transferz1020(1:Ntrans,3),y2tmp,Ntrans,p0,y)
	
	transferz1020_bar = y
	
end function transferz1020_bar


end module initial