!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module containing verious interpolation routines. See discription for more details !!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



module interpolation

use settings

!2 spline techniques, the first is slow but works for any input table, the second is fast but requires 
!equally spaced tables

contains

      subroutine splie2(x1a,x2a,ya,m,n,y2a)

      !splie2 use: given an array of x1a data of length m, and an array
      !of x2a data of length n, this routine computes the 2nd
      !derivatives, y2a, at each x1a,x2a data point.  thus y2a has
      !dimensions y2a(m,n).  natural spline fitting is assumed.

      !this routine called once, prior to using routine splin2, as a set
      !up for using routine splin2, which performs the actual
      !interpolation.

      !uses routines: spline

      !important note: the x1a and x2a data values must both be in
      !ascending order or the interpolation will fail

      implicit double precision (a-h,o-z)
      integer, parameter :: nn=600
      real(mcp) :: x1a(m),x2a(n),ya(m,n),y2a(m,n),ytmp(nn),y2tmp(nn)

      do 13 j=1,m
        do 11 k=1,n
          ytmp(k)=ya(j,k)
11      continue
        call spline(x2a,ytmp,n,1.d30,1.d30,y2tmp)
        do 12 k=1,n
          y2a(j,k)=y2tmp(k)
12      continue
13    continue
      return
      
      end subroutine splie2

      subroutine splin2(x1a,x2a,ya,y2a,m,n,x1,x2,y)

      !splin2 use: given an array of x1a data of length m, an array of
      !x2a data of length n, and an array of 2nd derivatives, y2a at each
      !x1a,x2a data point, dimensioned y2a(m,n), this routine performs 2d
      !interpolation, returning the interpolated value y at user input
      !values x1 and x2.  natural spline fitting is assumed.

      !uses routines: splint, spline

      !important note: the x1a and x2a data values must both be in
      !ascending order or the interpolation will fail


      implicit double precision (a-h,o-z)
      integer, parameter :: nn=600
      real(mcp) :: x1a(m),x2a(n),ya(m,n),y2a(m,n),ytmp(nn),y2tmp(nn),yytmp(nn)
      
      do 12 j=1,m
        do 11 k=1,n
          ytmp(k)=ya(j,k)
          y2tmp(k)=y2a(j,k)
11      continue
        call splint(x2a,ytmp,y2tmp,n,x2,yytmp(j))
12    continue
      call spline(x1a,yytmp,m,1.d30,1.d30,y2tmp)
      call splint(x1a,yytmp,y2tmp,m,x1,y)
      
      return
      
      end subroutine splin2

      subroutine spline(x,y,n,yp1,ypn,y2)

     !spline use: given an 1d array of x data and an array of y data,
     !both of length n, this routine computes the 2nd derivatives, y2 at
     !each x data point.  the user needs to specify the values of yp1
     !and yp2, which flags how the y2 are computed at the edges.  for
     !natural spline fitting (recommended), set yp1 and ypn to numbers
     !greater than 1.0e+30.

     !this routine called once, prior to using routine splint, as a set
     !up for using routine splint, which performs the actual
     !interpolation

     !important note: the x data values in array x must be in ascending
     !order or the interpolation will fail

      implicit double precision (a-h,o-z)
      integer, parameter :: nmax=600
      real(mcp) :: x(n),y(n),y2(n),u(nmax)

     !if yp1>1.0e+30 use natural spline, otherwise estimate y2 at the
     !first point

      if (yp1.gt..99d30) then
        y2(1)=0.d0
        u(1)=0.d0
      else
        y2(1)=-0.5d0
        u(1)=(3.d0/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif

     !store intermediate values of terms in the expansion series

      do 11 i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.d0
        y2(i)=(sig-1.d0)/p
        u(i)=(6.*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1)) &
           /(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
11    continue

     !if ypn>1.0e+30 use natural spline, otherwise estimate y2 at the
     !last point point

      if (ypn.gt..99d30) then
        qn=0.d0
        un=0.d0
      else
        qn=0.5d0
        un=(3.d0/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.d0)

     !compute the y2 from the 2nd order expansion series

      do 12 k=n-1,1,-1
        y2(k)=y2(k)*y2(k+1)+u(k)
12    continue

      return
      end subroutine spline


      subroutine splint(xa,ya,y2a,n,x,y)

     !splint use: given an 1d array of xa data, an array of ya data, and
     !an array of the 2nd derivatives y2a, all of length n, this routine
     !performs cubic spline interpolation, returning the interpolated
     !value y at the user input value x.  the y2a are computed in
     !routine spline, which is called once before calling splint.

     !important note: the x data values in array x must be in ascending
     !order or the interpolation will fail

      implicit double precision (a-h,o-z)
      real(mcp) :: xa(n),ya(n),y2a(n)

      klo=1
      khi=n

     !determine the indices of array xa that bracket the input x value

1     if (khi-klo.gt.1) then
        k=(khi+klo)/2
        if(xa(k).gt.x)then
          khi=k
        else
          klo=k
        endif
      goto 1
      endif

     !determine the finite difference along the x dimension

      h=xa(khi)-xa(klo)
      if (h.eq.0.) stop 'bad xa input in routine spline.'

     !interpolate

      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
      y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.d0

      return
      end subroutine splint
      
	  subroutine rec_interp1d(x0, dx, ytab, Nx, x, y) 
	  integer :: Nx
	  integer :: ix
	  real(mcp) :: frac
	  real(mcp) :: x0, dx, x
	  real(mcp) :: ytab(0:Nx-1), y
	  
	  ! Check if in range 
	  if (dx > 0 .and. (x<x0 .or. x>x0+dx*(Nx-1))) then
	    
		stop "Error: rec_interp1d: x-value out of range in interpolation routine."
		 
	  endif
	  
	  if (dx < 0 .and. (x>x0 .or. x<x0+dx*(Nx-1))) then
		stop "Error: rec_interp1d: x-value out of range in interpolation routine."
		
	  endif
	
	  !Identify location to interpolate 
	  ix = int(floor((x-x0)/dx))
	  if (ix<1) ix=1 
	  if (ix>Nx-3) ix=Nx-3
	  frac = (x-x0)/dx-ix
	  !ytab = ytab + ix-1
	
	  ! Return value 
	  
	  y = -ytab(ix-1)*frac*(1.-frac)*(2.-frac)/6.+ytab(ix)*(1.+frac)*(1.-frac)*(2.-frac)/2. &
			 +ytab(ix+1)*(1.+frac)*frac*(2.-frac)/2.-ytab(ix+2)*(1.+frac)*frac*(1.-frac)/6.
	  return
	  end subroutine rec_interp1d

!************************************************************************************
!Interpolation routine for 2-D table. Uses bicubic interpolation assuming
!uniformly spaced x1 and x2-values.
!*************************************************************************************/

		subroutine rec_interp2d(x10, dx1, x20, dx2, ytab, Nx1, Nx2, x1, x2, y2)
		
		integer :: j
		integer :: Nx1, Nx2
		integer :: ix1
		real(mcp) :: frac1, x10, dx1, x20, dx2, y1, y2, x2, x1
		real(mcp) :: temp(4), ytab(0:Nx1-1,0:Nx2-1), ytab2(0:Nx2-1)
		
		!Check if in range in x1
		if (x1<x10 .or. x1>x10+dx1*(Nx1-1)) then
		stop "Error: rec_interp2d: x1-value out of range in interpolation routine."
		
		endif
		
		!Identify location to interpolate in x1 
		ix1 = int(floor((x1-x10)/dx1))
		if (ix1<1) ix1=1
		if (ix1>Nx1-3) ix1=Nx1-3
		frac1 = (x1-x10)/dx1-ix1
		!ytab = ytab+ix1-1  
		
		! Get values interpolated over x2 at the 4 neighboring points in x1 */
		do j=0, 3
			do i = 0, Nx2-1
			ytab2(i) = ytab((ix1-1+j),i)
			enddo
			call rec_interp1d(x20,dx2,ytab2,Nx2,x2,y1)
			temp(j+1) = y1
		enddo 
		
		y2 = -temp(1)*frac1*(1.-frac1)*(2.-frac1)/6. +temp(2)*(1.+frac1)*(1.-frac1)*(2.-frac1)/2. &
			 +temp(3)*(1.+frac1)*frac1*(2.-frac1)/2. +temp(4)*(1.+frac1)*frac1*(frac1-1.)/6.
		return
		end subroutine rec_interp2d


end module interpolation