module mpi_params 

#ifdef MPI
use mpi 
#endif
!use CAMB
use settings

implicit none 
	   
#ifdef MPI	   
   Type mpi_pars
		 integer :: rank, size, ierror, tag, status(MPI_STATUS_SIZE)
		 integer :: steps, res, istart, iend
		 integer :: root
   end Type mpi_pars
   
   type(mpi_pars) :: omp

   integer, allocatable, dimension(:)   :: displs, recvcounts 
   real(mcp), allocatable  :: proc_contrib(:,:,:) 
   
		
   
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module to initialize mpi settings and determine computational load per CPU!!!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	
	contains     
	
	subroutine init_mpi_params(nn) 
	
		integer, intent(in) :: nn 
		integer :: i, slen 
		
		if (allocated(displs)) deallocate(displs)
		if (allocated(recvcounts)) deallocate(recvcounts)
		
		! Determine how many points to handle with each proc 
		omp%root = 0
		call MPI_COMM_SIZE(MPI_COMM_WORLD, omp%size, omp%ierror)
        call MPI_COMM_RANK(MPI_COMM_WORLD, omp%rank, omp%ierror)
        
!	     omp%steps = floor(real(nn/omp%size))
	 	
	 	if ( mod(nn,omp%size)==0 ) then 
			omp%steps = nn/omp%size 
	 	else 
			omp%steps = (nn-mod(nn,omp%size))/omp%size 
		
			if (omp%size-1 == omp%rank ) omp%steps = nn - & 
			omp%steps*(omp%size-1) 
			
	 	end if 
	 	
		
		! Determine start and end index for this proc's points 
	    omp%istart = omp%rank*(nn-mod(nn,omp%size))/omp%size+1
	    omp%iend = (omp%rank+1)*(nn-mod(nn,omp%size))/omp%size
	     
		!omp%res = nn - omp%size*omp%steps
	    if((omp%rank+1) .eq. omp%size) then
	        omp%iend  = omp%iend  + mod(nn,omp%size)
	    endif

		
		!allocate(proc_contrib(omp%steps,Nvbc,Nk)) 
		
		!print *, 'about to allocate displs' 
		
		allocate(displs(omp%size),source=(/(i*(nn-mod(nn,omp%size))/omp%size,i=0,omp%size-1)/)) 
		allocate(recvcounts(omp%size),source=(/((nn-mod(nn,omp%size))/omp%size,i=1,omp%size)/)) 
		recvcounts(omp%size)=(nn - omp%steps*(omp%size-1)) 
!		
		if (omp%size-1 == omp%rank ) recvcounts(omp%size) = (omp%iend-omp%istart+1) 
		
	end subroutine init_mpi_params 
#endif

end module mpi_params