!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module to compute perturbations for every k, vbc and a to 1st and 2nd order!!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module perturbations

use settings
use readfiles
use background
use bkgfunctions
use mpi_params
use initial
use ode_solver
use coefficients

implicit none

real(mcp) :: xe_finalintktab(Na, Nvbc), Tg_finalintktab(Na, Nvbc)
real(mcp), public :: public_vbc,tgasPUBLIC
real(mcp), public :: public_k0 
real(mcp), public :: public_a

real(mcp), public :: Prob1dtab(Nvbc), Prob3dtab(Nvbc)

!2nd order tabs:
real(mcp), public :: av_db2tab(Na), db2_vbcavtab(Na, Nvbc)
real(mcp), public :: de2tab(Na,Nvbc), Nde2tab(Na,Nvbc)
real(mcp), public :: dT2tab(Na,Nvbc), NdT2tab(Na,Nvbc)
real(mcp), public :: T21quadavtab(Na, Nvbc)



type(pert) :: delta
	
public delta
public xe_finalintktab, Tg_finalintktab

contains 
	
	real(mcp) function Prob3D(vbc)
	
		real(mcp) :: vbc
	
		Prob3D =((0.5/pi)**(1.5d0)/(sigma1D)**3)*4.0d0*pi*vbc**2*Exp(-0.5d0*(vbc/sigma1D)**2)
	
	end function Prob3D
    
    real(mcp) function prob1D(vdotk)
	
		real(mcp) :: vdotk
	
		prob1D = Sqrt(2.d0/Pi)/sigma1D*Exp(-0.5d0*(vdotk/sigma1D)**2)
	
    end function prob1D
    
    subroutine set_probtabs
    
		real(mcp) :: Dvbc
		integer :: i
		
		Dvbc = vbcmax/(Nvbc - 1.0d0)
		
		do i = 1, Nvbc	
			Prob1dtab(i) = Prob1D((i-1)*Dvbc)*Dvbc
			Prob3dtab(i) = Prob3D((i-1)*Dvbc)*Dvbc
			!if (omp%rank .eq. my_mpi_root) write(*,*) Prob1dtab(i)
		enddo 
		
		Prob1dtab(1) = 0.5d0*Prob1dtab(1)
		Prob3dtab(1) = 0.5d0*Prob3dtab(1)
		
		Prob1dtab(Nvbc) = 0.5d0*Prob1dtab(Nvbc)
		Prob1dtab(Nvbc) = 0.5d0*Prob1dtab(Nvbc)
        
    end subroutine set_probtabs
    
    
    
    
    real(mcp) function Cpeebles_full(a,delb,delvb)
    real(mcp) :: a
    real(mcp) :: denom
    real(mcp) :: delb,delvb


    !note that here delb and delv are dummy variables, otherwise one would have to consider 
    !complex variables
   
    
    denom = betaB(Tcmb(a)) + &
              0.25d0*RCBparams%L2s + 0.75d0*Rlymana_full(a,delb,delvb)
    CPeebles_full = 1.d0-betaB(Tcmb(a))/denom
    
    
    end function
    
    real(mcp) function Rlymana_full(a,delb,delvb)
    
    real(mcp) :: delb,delvb
    real(mcp) :: a    
    real(mcp) :: denom
     
    denom = (nhbar(a)*(1.d0+delb)*(1.d0 - xebar(a))) 
    
    Rlymana_full=1.5109848E-4*(Hubble(a)+delvb/3.d0)/denom  
   
    end function Rlymana_full 

    real(mcp) function xedot_full(a,delb,delvb,delT,dele)
    !can always decide to make a type for these later
    real(mcp) :: delb,delvb,delT,dele
    real(mcp) :: a    
    !write(*,*) "Now inside xedot"
    !xedot = Cpeebles*nhbar*alphaB*xe**2:
    xedot_full = -Cpeebles_full(a,delb,delvb)*(1.d0+delb)* &
    				nhbar(a)*alphaB(Tcmb(a),Tgasbar(a)*(1.d0+delT))* &
    				xebar(a)**2.d0*(1.d0+dele)**2.d0/derCMBparams%kmpermpc 
    				
    if (RCBparams%DMAN_on) then 
    xedot_full = xedot_full + Urate(a)*(1-xebar(a)*(1.d0+dele))* &
    				(1.d0+4.d0*(1.d0-Cpeebles_full(a,delb,delvb))/3.0d0)/(nhbar(a)*(1.d0+delb))/3.0d0
    endif 
    
    !write(*,*) "Now outside xedot"
    end function xedot_full



    real(mcp) function xe_quadratic(a,delb,delvb,delT,dele)
    real(mcp) :: delb,delvb,delT,dele
	real(mcp) :: a    
    !for more accuracy make this smaller:
    real(mcp) :: eps = 0.01d0
    real(mcp) :: Fmeps
    real(mcp) :: Fpeps
    real(mcp) :: Fzero
    
    !write(*,*) "Now inside xe_q"
    
    Fpeps = xedot_full(a,eps*delb,eps*delvb,eps*delT,eps*dele)
    Fmeps = xedot_full(a,-eps*delb,-eps*delvb,-eps*delT,-eps*dele)
    Fzero = xedot_full(a,0.d0,0.d0,0.d0,0.d0)
    xe_quadratic= (1.d0/2.d0)*((Fpeps+Fmeps) -2.0d0*Fzero)/xebar(a)/eps**2.d0
     !write(*,*) "Now leaving"
    end function xe_quadratic

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! Second order de and dT source terms !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


    
    subroutine compute_xe_quadratic(fluc,xe_quadratictab)
 
    type(fluctuations), intent(in) :: fluc
    real(mcp) :: xe_quadratictab(Na, Nvbc, Nk)
    real(mcp), allocatable :: my_array(:,:,:)
    integer :: i,j,l,jmin,jmax, int_run, tag, int_s
    real(mcp) :: ReTabXeDot
    real(mcp) :: ImTabXeDot
    real(mcp) :: Redelb,Redelvb,RedelT,Redele
    real(mcp) :: Imdelb,Imdelvb,ImdelT,Imdele
    real(mcp) :: a  
#ifdef MPI 
	real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na*Nvbc*Nk)

    !call set_2tabs()
    
    CALL init_mpi_params(Na)
    
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)*Nvbc*Nk
        displs(i) = displs(i)*Nvbc*Nk
    enddo
    
    allocate(temp(recvcounts(omp%rank+1)))
    allocate(proc_contrib(omp%steps,Nvbc,Nk)) 
    
    int_s = 1


    if (omp%rank .eq. my_mpi_root) write(*, '(50a)') "**********************************************100%"
#else
    write(*, '(50a)') "**********************************************100%"
#endif
#ifdef MPI 
    do j = omp%istart, omp%iend
#else
	do j = 1, Na
#endif
    	do i = 1, Nvbc
    		do l = 1, Nk
!one can uncomment this (but needs come extra mods) to use read deltas from file (i.e. precomputed)    		
!    		int_run = (j - 1)*Nvbc*Nk + (i - 1)*Nk + l
!			a = fluc%bar(int_run,1)
!    		Redelb = fluc%bar(int_run,4)   		
!    		Redelvb = fluc%vbar(int_run,4)
!    		RedelT = fluc%Tgas(int_run,4)
!    		Redele = fluc%xe(int_run,4)
!    		Imdelb = fluc%bar(int_run,5)
!    		Imdelvb = fluc%vbar(int_run,5)
!    		ImdelT = fluc%Tgas(int_run,5)
!    		Imdele = fluc%xe(int_run,5)
!if using above, comment these out
    		Redelb = delta%Rebar(j,i,l)   		
    		Redelvb = delta%Revbar(j,i,l)
    		RedelT = delta%ReTgas(j,i,l)
    		Redele = delta%Rexe(j,i,l)
    		Imdelb = delta%Imbar(j,i,l)
    		Imdelvb = delta%Imvbar(j,i,l)
    		ImdelT = delta%ImTgas(j,i,l)
    		Imdele = delta%Imxe(j,i,l)
    		
    		
			a = atab(j)
			!write(*,*) a, j
    		ReTabXeDot = xe_quadratic(a,Redelb,Redelvb,RedelT,Redele)
    		ImTabXeDot = xe_quadratic(a,Imdelb,Imdelvb,ImdelT,Imdele)
    		xe_quadratictab(j,i,l) = ReTabXeDot+ImTabXeDot
#ifdef MPI      		
            temp(int_s) = ReTabXeDot+ImTabXeDot
            int_s = int_s +1
#else
			xe_quadratictab(j,i,l) = ReTabXeDot+ImTabXeDot
			!write(*,*) xe_quadratictab(j,i,l)
#endif
    		if (feedback > 0 .and. i .eq. 30 .and. l .eq. 20) then
    		write(*,*) j,a, xe_quadratictab(j, i, l)
    		endif
    		enddo
    	enddo
    	if(mod(j,2) .eq. 0) write(*,'(a)',advance='no')  '*'
    enddo
#ifdef MPI   
    !making sure all proc. share the computed table:             
    call MPI_ALLGATHERV(temp, omp%steps*Nk*Nvbc, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the marix 
    int_s = 1
    
    do j = 1, Na
        do i = 1, Nvbc
        	do l = 1, Nk
                xe_quadratictab(j,i,l) = tottemp(int_s)
    			int_s = int_s + 1
    		enddo 
    	enddo 
    enddo 

    deallocate(proc_contrib)
    deallocate(temp)
    !for now

    deallocate(displs)
    deallocate(recvcounts)
    
	if(omp%rank .eq. my_mpi_root) write(*,*) ""
#else
	write(*,*) ""
#endif
    
    end subroutine compute_xe_quadratic
    
    subroutine compute_Tgas_quadratic(fluc,Tg_quadratictab)
 
    type(fluctuations), intent(in) :: fluc
    real(mcp) :: Tg_quadratictab(Na, Nvbc, Nk)
    integer :: i,j,l,jmin,jmax, int_run, int_s
    real(mcp) :: Tga1, Tga2, Tga3
    real(mcp) :: Redelb,Redelvb,RedelT,Redele,Redelc
    real(mcp) :: Imdelb,Imdelvb,ImdelT,Imdele,Imdelc
    real(mcp) :: a  
#ifdef MPI   
	real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na*Nvbc*Nk)	

    CALL init_mpi_params(Na)
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)*Nvbc*Nk
        displs(i) = displs(i)*Nvbc*Nk
    enddo
    allocate(temp(recvcounts(omp%rank+1)))
    allocate(proc_contrib(omp%steps,Nvbc,Nk)) 
    int_s = 1
    
    do j = omp%istart, omp%iend 
#else
	do j = 1, Na
#endif
  		do i = 1, Nvbc
    		do l = 1, Nk
!one can uncomment this (but needs some extra mods) to use read deltas from file (i.e. precomputed)        		
!    		int_run = (j - 1)*Nvbc*Nk + (i - 1)*Nk + l
!    		a = fluc%bar(int_run,1)
!    		Redelb = fluc%bar(int_run,4)   		
!    		Redelvb = fluc%vbar(int_run,4)
!    		RedelT = fluc%Tgas(int_run,4)
!    		Redele = fluc%xe(int_run,4)
!    		Imdelb = fluc%bar(int_run,5)
!    		Imdelvb = fluc%vbar(int_run,5)
!    		ImdelT = fluc%Tgas(int_run,5)
!    		Imdele = fluc%xe(int_run,5)
!if using above, comment these out    		
    		Redelb = delta%Rebar(j,i,l)   		
    		Redelvb = delta%Revbar(j,i,l)
    		RedelT = delta%ReTgas(j,i,l)
    		Redele = delta%Rexe(j,i,l)
    		Imdelb = delta%Imbar(j,i,l)
    		Imdelvb = delta%Imvbar(j,i,l)
    		ImdelT = delta%ImTgas(j,i,l)
    		Imdele = delta%Imxe(j,i,l)
    		
    		if (RCBparams%DMAN_on ) then
    		Redelc = delta%Recdm(j,i,l)
    		Imdelc = delta%Imcdm(j,i,l)
    		endif
			a = atab(j)
    		
    		Tga1 = 2.d0/3.d0*(Redelvb*Redelb-Redelvb*RedelT+Imdelvb*Imdelb-Imdelvb*ImdelT)
    		Tga2 = Gcompton(a)*(Redele*RedelT+Imdele*ImdelT)
    		
    		!dark matter:
    		if (RCBparams%DMAN_on) then 
    			!should not have any UNITS, hence, has to be divided by Tgas
    			!but Tgasbar, has to be the modified one. WO dm anahilation, it divides out, here it
    			!does not
    			Tga3 = 2.0d0*Gchi(a)*(Redelc*Redelb- Imdelc*Imdelb + xebar(a)*(2.0d0*Redelc*Redele+ &
    					2.0d0*Imdelc*Imdele + Redelb*Redele+Imdelb*Imdele)/(1.d0+2.d0*xebar(a)))/Tgas2tab(j)
    		else 
    			Tga3 = 0.0d0
    		endif
    		
#ifdef MPI       		
    		temp(int_s) = Tga1-Tga2+Tga3
    		int_s = int_s +1
    		Tg_quadratictab(j, i, l) = Tga1-Tga2+Tga3
#else 
    		Tg_quadratictab(j,i,l) = Tga1-Tga2+Tga3
#endif    		
            if (feedback > 1 .and. i .eq. 2 .and. l .eq. 1) then
    		write(*,*) a, Tg_quadratictab(j, i, l)
    		endif
			   		
    		enddo
    	enddo 
    enddo 
#ifdef MPI    
    !making sure all proc. share the computed table:             
    call MPI_ALLGATHERV(temp, omp%steps*Nk*Nvbc, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the matrix 
    int_s = 1
    
    do j = 1, Na
        do i = 1, Nvbc
        	do l = 1, Nk
                Tg_quadratictab(j,i,l) = tottemp(int_s)
    			int_s = int_s + 1
    		enddo 
    	enddo 
    enddo 
    deallocate(proc_contrib)
    deallocate(temp)
    !for now (jun 20, 2014, added allocation check inside mpi_params routine, so should be taken care off
    !in case one forgets. Leave these for now. )
  
    deallocate(displs)
    deallocate(recvcounts)
#endif    
    end subroutine compute_Tgas_quadratic

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! The k Integrals !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   
    subroutine kintegral_desource(xe_quadratictab,xe_sourceintktab)
    
    real(mcp) :: xe_quadratictab(Na, Nvbc, Nk)    
    real(mcp) :: xe_sourceintktab(Na, Nvbc)
    integer :: j, i, l, jmax, jmin, int_s
    real(mcp) :: kint_temp

#ifdef MPI
    real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na*Nvbc)
		
	if (omp%rank .eq. my_mpi_root) write(*, '(50a)') "**********************************************100%" 
	
    CALL init_mpi_params(Na)
    
    kint_temp = 0.d0
    
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)*Nvbc
        displs(i) = displs(i)*Nvbc
    enddo
    
    
    allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
#else
    write(*, '(50a)') "**********************************************100%" 
#endif


#ifdef MPI
    do j = omp%istart, omp%iend 
#else 
	do j = 1, Na
#endif
    	do i = 1, Nvbc
    	kint_temp = 0.d0
    		do l = 1, Nk
    		kint_temp = kint_temp + xe_quadratictab(j, i, l)*Dlogk   
    		enddo
#ifdef MPI 
		xe_sourceintktab(j, i) = kint_temp
    	temp(int_s) = kint_temp
    	int_s = int_s+1
#else
		xe_sourceintktab(j, i) = kint_temp	
#endif
	    if (feedback > 1 .and. i .eq. 2) then
		write(*,*) "K1",xe_sourceintktab(j, i)
		endif
		
    	enddo
    enddo    
#ifdef MPI    
    !making sure all proc. share the computed table:             
    call MPI_ALLGATHERV(temp, omp%steps*Nvbc, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the matrix 
    int_s = 1
    
    do j = 1, Na
        do i = 1, Nvbc	
           xe_sourceintktab(j,i) = tottemp(int_s)
    	   int_s = int_s + 1 
    	enddo 
    enddo 
    !deallocate(proc_contrib)
    deallocate(temp)
    
    deallocate(displs)
    deallocate(recvcounts)
#endif    
    end subroutine kintegral_desource 
    
    subroutine kintegral_dTgsource(Tg_quadratictab,Tg_sourceintktab)
    
    real(mcp) :: Tg_quadratictab(Na, Nvbc, Nk)    
    real(mcp) :: Tg_sourceintktab(Na, Nvbc)
    integer :: j, i, l, jmax, jmin, int_s
    real(mcp) :: kint_temp
#ifdef MPI    
    real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na*Nvbc)

    CALL init_mpi_params(Na)
    
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)*Nvbc
        displs(i) = displs(i)*Nvbc
    enddo
       
    allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
    
    do j = omp%istart, omp%iend 
#else
	do j = 1, Na
#endif
    	do i = 1, Nvbc
    	kint_temp = 0.d0
    		do l = 1, Nk
    		kint_temp = kint_temp + Tg_quadratictab(j, i, l)*Dlogk   
    		enddo
#ifdef MPI
		Tg_sourceintktab(j, i) = kint_temp
    	temp(int_s) = kint_temp
    	int_s = int_s+1
#else
		Tg_sourceintktab(j, i) = kint_temp	
#endif		
	    if (feedback > 1 .and. i .eq. 2) then
		write(*,*) "K2",Tg_sourceintktab(j, i)
		endif
		
    	enddo
    enddo    
#ifdef MPI    
    !making sure all proc. share the computed table:             
    call MPI_ALLGATHERV(temp, omp%steps*Nvbc, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the matrix 
    int_s = 1
    
    do j = 1, Na
        do i = 1, Nvbc	
           Tg_sourceintktab(j,i) = tottemp(int_s)
    	   int_s = int_s + 1 
    	enddo 
    enddo 
    !deallocate(proc_contrib)
    deallocate(temp)
    
    deallocate(displs)
    deallocate(recvcounts)
#endif    
    end subroutine kintegral_dTgsource 
    
    subroutine kintegral_db2(db2_intktab)
   
    integer :: j, i, l, jmax, jmin, int_s
    real(mcp) :: kint_temp
    real(mcp) :: absdb2
    real(mcp) :: db2_intktab(Na, Nvbc)
#ifdef MPI    
    real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na*Nvbc)
    
    CALL init_mpi_params(Na)
    
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)*Nvbc
        displs(i) = displs(i)*Nvbc
    enddo
       
    allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
    
    do j = omp%istart, omp%iend 
#else
	do j = 1, Na
#endif
    	do i = 1, Nvbc
    	kint_temp = 0.d0
    		do l = 1, Nk
    		absdb2 = delta%Rebar(j,i,l)**2 + delta%Imbar(j,i,l)**2
    		kint_temp = kint_temp + absdb2*Dlogk   
    		enddo
#ifdef MPI   
		db2_intktab(j, i) = kint_temp
    	temp(int_s) = kint_temp
    	int_s = int_s+1
#else
		db2_intktab(j, i) = kint_temp	
#endif		
	    if (feedback > 1 .and. i .eq. 20) then
		write(*,*) "K3", db2_intktab(j, i)
		endif
		
    	enddo
    enddo    
#ifdef MPI    
    !making sure all proc. share the computed table:             
    call MPI_ALLGATHERV(temp, omp%steps*Nvbc, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the matrix 
    int_s = 1
    
    do j = 1, Na
        do i = 1, Nvbc	
           db2_intktab(j, i) = tottemp(int_s)
    	   int_s = int_s + 1 
    	enddo 
    enddo 
    !deallocate(proc_contrib)
    deallocate(temp)
    
    deallocate(displs)
    deallocate(recvcounts)
#endif    
    end subroutine kintegral_db2 
    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! The angle Integrals !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    subroutine vbcint_desource(xe_sourceintktab)
  
    real(mcp) :: xe_sourceintktab(Na, Nvbc)
    integer :: j, i,l, jmax, jmin, int_s
    real(mcp) :: vbcint_temp
#ifdef MPI    
    real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na*Nvbc)
 
    CALL init_mpi_params(Na)
    
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)*Nvbc
        displs(i) = displs(i)*Nvbc
    enddo
        
    allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
    
    do j = omp%istart, omp%iend 
#else
	do j = 1, Na
#endif
    	do i = 1, Nvbc
    	vbcint_temp = 0.d0
    		do l = 1, i
    		vbcint_temp = vbcint_temp + xe_sourceintktab(j, l)   
    		
    		enddo
#ifdef MPI
		xe_finalintktab(j, i) = vbcint_temp/i
    	temp(int_s) = vbcint_temp/i
    	int_s = int_s +1
#else
		xe_finalintktab(j, i) = vbcint_temp/i	
#endif	    				
    	enddo
    	if (feedback > 1) then
    	write(*,*) "V1", xe_finalintktab(j, 20)
    	write(*,*) "V1b",xe_sourceintktab(j, 20)
    	endif
    	 
    enddo    
#ifdef MPI    
    call MPI_ALLGATHERV(temp, omp%steps*Nvbc, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the marix 
    int_s = 1
    
    do j = 1, Na
        do i = 1, Nvbc	
           xe_finalintktab(j,i) = tottemp(int_s)
    	   int_s = int_s + 1 
    	enddo 
    enddo 
    deallocate(temp)
    
    deallocate(displs)
    deallocate(recvcounts)
    
#endif    
    end subroutine vbcint_desource
    
    subroutine vbcint_dTgsource(Tg_sourceintktab)
  
    real(mcp) :: Tg_sourceintktab(Na, Nvbc)
    integer :: j, i,l, jmax, jmin, int_s
    real(mcp) :: vbcint_temp
#ifdef MPI    
    real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na*Nvbc)
 
    CALL init_mpi_params(Na)
    
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)*Nvbc
        displs(i) = displs(i)*Nvbc
    enddo
    
    
    allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
    
    do j = omp%istart, omp%iend 
#else
	do j = 1, Na
#endif
    	do i = 1, Nvbc
    	vbcint_temp = 0.d0
    		do l = 1, i
    		vbcint_temp = vbcint_temp + Tg_sourceintktab(j, l)   
    		
    		enddo
#ifdef MPI 
        Tg_finalintktab(j, i) = vbcint_temp/i
    	temp(int_s) = vbcint_temp/i
        int_s = int_s +1
#else
		Tg_finalintktab(j, i) = vbcint_temp/i	
#endif				
    	enddo
    	if (feedback > 1) then
    	write(*,*) "V2", Tg_finalintktab(j, 2)
    	endif
    if(mod(j,2) .eq. 0) write(*,'(a)',advance='no')  '*'	 
    enddo    
#ifdef MPI    
    call MPI_ALLGATHERV(temp, omp%steps*Nvbc, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the marix 
    int_s = 1
    
    do j = 1, Na
        do i = 1, Nvbc	
           Tg_finalintktab(j,i) = tottemp(int_s)
    	   int_s = int_s + 1 
    	enddo 
    enddo 

    deallocate(temp)
    deallocate(displs)
    deallocate(recvcounts)
    if(omp%rank .eq. my_mpi_root) write(*,*) ""
#else
	write(*,*) ""
#endif
    
    end subroutine vbcint_dTgsource
    
    
    subroutine vbcint_db2(db2_intktab)
    
    real(mcp) :: db2_intktab(Na, Nvbc)
    integer :: j, i,l, jmax, jmin, int_s
    real(mcp) :: vbcint_temp
#ifdef MPI
    real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na*Nvbc)
 
    CALL init_mpi_params(Na)
    
    vbcint_temp = 0.d0
    
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)*Nvbc
        displs(i) = displs(i)*Nvbc
    enddo
    
    
    allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
    
    do j = omp%istart, omp%iend 
#else
	do j = 1, Na
#endif
    	do i = 1, Nvbc
    	vbcint_temp = 0.d0
    		do l = 1, i
    		vbcint_temp = vbcint_temp + db2_intktab(j, l)   
    		
    		enddo
#ifdef MPI
        db2_vbcavtab(j, i) = vbcint_temp/i
    	temp(int_s) = vbcint_temp/i
    	int_s = int_s +1
#else
		db2_vbcavtab(j, i) = vbcint_temp/i	
#endif	    
    	enddo
    	if (feedback > 1) then
    	write(*,*) "V3", db2_vbcavtab(j, 2)
    	endif
    	 
    enddo    
#ifdef MPI    
    call MPI_ALLGATHERV(temp, omp%steps*Nvbc, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the marix 
    int_s = 1
    
    do j = 1, Na
        do i = 1, Nvbc	
           db2_vbcavtab(j,i) = tottemp(int_s)
    	   int_s = int_s + 1 
    	enddo 
    enddo 
    deallocate(temp)
    
    deallocate(displs)
    deallocate(recvcounts)
#endif    
    
    end subroutine vbcint_db2
    
    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! Averaging !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	
	subroutine av_db2(db2_intktab)
    
    real(mcp) :: db2_intktab(Na, Nvbc)
    integer :: j, i,l, jmax, jmin, int_s
    real(mcp) :: av_db2t_temp
#ifdef MPI
    real(mcp), allocatable :: temp(:)
	real(mcp) :: tottemp(Na)
    
    CALL init_mpi_params(Na)
    
    do i =1, omp%size
    	recvcounts(i) = recvcounts(i)
        displs(i) = displs(i)
    enddo
    
    
    allocate(temp(recvcounts(omp%rank+1)))
    int_s = 1
    
    do j = omp%istart, omp%iend 
#else
	do j = 1, Na
#endif
    av_db2t_temp = 0.d0
    		do i = 1, Nvbc    	
    			av_db2t_temp = av_db2t_temp + db2_intktab(j,i)*prob1Dtab(i)   
    		enddo
#ifdef MPI   
		av_db2tab(j) = av_db2t_temp	
    	temp(int_s) = av_db2t_temp
    	int_s = int_s +1
#else
		av_db2tab(j) = av_db2t_temp	
#endif
	       	
    	if (feedback > 1) then
    	write(*,*) "A1", av_db2tab(j)
    	endif
    	 
    enddo    
#ifdef MPI    
    call MPI_ALLGATHERV(temp, omp%steps, MPI_REAL8, &
                       tottemp, recvcounts, displs, MPI_Real8, &
                      MPI_COMM_WORLD, omp%ierror) 
         
    !now storing that in the marix 
    int_s = 1
    
    do j = 1, Na	
        av_db2tab(j) = tottemp(int_s)
    	int_s = int_s + 1 
    enddo 
    !renormalizing 
    do j = 1, Na
    	do i = 1, Nvbc
    	db2_vbcavtab(j,i) = db2_vbcavtab(j,i) - av_db2tab(j)
    	
    	enddo 
	enddo 
    
    deallocate(temp)
    
    deallocate(displs)
    deallocate(recvcounts)
#endif
    end subroutine av_db2

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! Source functions !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
        
    real(mcp) function sourcedT2(a,vbc)

    real(mcp) :: a, vbc
    real(mcp) :: ya(Na,Nvbc)
    real(mcp) :: y2aS(Na,Nvbc)
    real(mcp) :: yS
    real(mcp) :: dx1, dx2, x10, x20
    logical :: use_spline = .false.

    if (use_spline) then 
    	call splie2(atab,vbctab,Tg_finalintktab,Na,Nvbc,y2aS)
    	call splin2(atab,vbctab,Tg_finalintktab,y2aS,Na,Nvbc,a,vbc,yS)
    else
    	x10 = Log(atab(1))
    	x20 = vbctab(1)
    	dx1 = abs(Log(atab(1))-Log(atab(2)))
    	dx2 = abs(vbctab(1)-vbctab(2))
    
    	call rec_interp2d(x10, dx1, x20, dx2, Tg_finalintktab, Na, Nvbc, Log(a), vbc, yS)
    endif
    sourcedT2 = yS	
    
    return
    end function 
    
    real(mcp) function sourcede2(a,vbc)

    real(mcp) :: a, vbc
    real(mcp) :: ya(Na,Nvbc)
    real(mcp) :: y2aS(Na,Nvbc)
    real(mcp) :: yS
    real(mcp) :: dx1, dx2, x10, x20
    logical :: use_spline = .false.
    

    if (use_spline) then 
    	call splie2(atab,vbctab,xe_finalintktab,Na,Nvbc,y2aS)
    	call splin2(atab,vbctab,xe_finalintktab,y2aS,Na,Nvbc,a,vbc,yS)
    else
    	x10 = Log(atab(1))
    	x20 = vbctab(1)
    	dx1 = abs(Log(atab(1))-Log(atab(2)))
    	dx2 = abs(vbctab(1)-vbctab(2))
    
    	call rec_interp2d(x10, dx1, x20, dx2, xe_finalintktab, Na, Nvbc, Log(a), vbc, yS)
    endif
    sourcede2 = yS	
    
    return
    end function 

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! spline dT^2, db^2 and d21tot^2 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    

	real(mcp) function dT21Tgas(a,vbc)

    real(mcp) :: a, vbc
    real(mcp) :: ya(Na,Nvbc)
    real(mcp) :: y2aS(Na,Nvbc)
    real(mcp) :: yS
    real(mcp) :: dx1, dx2, x10, x20
    logical :: use_spline = .false.
    

    if (use_spline) then 
    	call splie2(atab,vbctab,NdT2tab,Na,Nvbc,y2aS)
    	call splin2(atab,vbctab,NdT2tab,y2aS,Na,Nvbc,a,vbc,yS)
    else
    	x10 = Log(atab(1))
    	x20 = vbctab(1)
    	dx1 = abs(Log(atab(1))-Log(atab(2)))
    	dx2 = abs(vbctab(1)-vbctab(2))
    
    	call rec_interp2d(x10, dx1, x20, dx2, NdT2tab, Na, Nvbc, Log(a), vbc, yS)
    endif
    
    dT21Tgas = coeffT(a)*yS	
    
    return
    end function dT21Tgas 


	real(mcp) function db2av(a,vbc)

    real(mcp) :: a, vbc
    real(mcp) :: ya(Na,Nvbc)
    real(mcp) :: y2aS(Na,Nvbc)
    real(mcp) :: yS
    real(mcp) :: dx1, dx2, x10, x20
    logical :: use_spline = .false.

    if (use_spline) then 
    	call splie2(atab,vbctab,db2_vbcavtab,Na,Nvbc,y2aS)
    	call splin2(atab,vbctab,db2_vbcavtab,y2aS,Na,Nvbc,a,vbc,yS)
    else
    	x10 = Log(atab(1))
    	x20 = vbctab(1)
    	dx1 = abs(Log(atab(1))-Log(atab(2)))
    	dx2 = abs(vbctab(1)-vbctab(2))
    
    	call rec_interp2d(x10, dx1, x20, dx2, db2_vbcavtab, Na, Nvbc, Log(a), vbc, yS)
    endif 
    db2av = yS	
    
    return
    end function db2av 
    
    real(mcp) function T21tot(a,vbc)

    real(mcp) :: a, vbc
    real(mcp) :: ya(Na,Nvbc)
    real(mcp) :: y2aS(Na,Nvbc)
    real(mcp) :: yS
    real(mcp) :: dx1, dx2, x10, x20
    logical :: use_spline = .false.

    if (use_spline) then 
    	call splie2(atab,vbctab,T21quadavtab,Na,Nvbc,y2aS)
    	call splin2(atab,vbctab,T21quadavtab,y2aS,Na,Nvbc,a,vbc,yS)
    else
    	x10 = Log(atab(1))
    	x20 = vbctab(1)
    	dx1 = abs(Log(atab(1))-Log(atab(2)))
    	dx2 = abs(vbctab(1)-vbctab(2))
    
    	call rec_interp2d(x10, dx1, x20, dx2, T21quadavtab, Na, Nvbc, Log(a), vbc, yS)
    endif
    
    T21tot = yS + dT21Tgas(a,vbc)
    
    return
    end function T21tot 
    
    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! derivatives !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  

	real(mcp) function db2avDer(a,vbc)
	real(mcp) :: a, vbc
	real(mcp) :: eps = 0.0001d0
	
	if (vbc .eq. 0.0d0) then 
	db2avDer = (db2av(a,vbc+eps)-db2av(a,vbc))/(eps)
	elseif (vbc .eq. vbcmax) then 
	db2avDer = (db2av(a,vbc)-db2av(a,vbc-eps))/(eps)
	else
	db2avDer = (db2av(a,vbc+eps)-db2av(a,vbc-eps))/(2.0d0*eps)
	endif 
	
	end function db2avDer
	
	real(mcp) function dT2Der(a,vbc)
	real(mcp) :: a, vbc
	real(mcp) :: eps = 0.0001d0
	
	if (vbc .eq. 0.0d0) then 
	dT2Der = (dT21Tgas(a,vbc+eps)-dT21Tgas(a,vbc))/(eps)
	elseif (vbc .eq. vbcmax) then 
	dT2Der = (dT21Tgas(a,vbc)-dT21Tgas(a,vbc-eps))/(eps)
	else
	dT2Der = (dT21Tgas(a,vbc+eps)-dT21Tgas(a,vbc-eps))/(2.0d0*eps)
	endif 
	
	end function dT2Der

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! 21cm brightness  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	subroutine compute_21IItot

	real(mcp) :: T21quadtab(Na,Nvbc,Nk)
	real(mcp) :: T21quadintktab(Na, Nvbc)
	real(mcp) :: T21Temp(Na)
	integer :: i, j, l
	
	real(mcp) :: kint_temp, vbcint_temp
	
	!defining the table
	do i = 1, Na
		do j = 1, Nvbc
			do l = 1, Nk
			
			T21quadtab(i,j,l) = coeffHH(atab(i))*(delta%Rebar(i,j,l)**2+delta%Imbar(i,j,l)**2)+ &
								coeffTT(atab(i))*(delta%ReTgas(i,j,l)**2+delta%ImTgas(i,j,l)**2)+ &
								coeffHT(atab(i))*(delta%ReTgas(i,j,l)*delta%Rebar(i,j,l) +  & 
								delta%Imbar(i,j,l)*delta%ImTgas(i,j,l))
			enddo 
		enddo
	enddo 
	!integrating over k
	do i = 1, Na
		do j = 1, Nvbc
		kint_temp = 0.0d0
			do l = 1, Nk
			kint_temp = kint_temp + T21quadtab(i,j,l)
			enddo 
		T21quadintktab(i,j) = kint_temp*Dlogk
		enddo
		
	enddo  
	!angle integral
	do i = 1, Na
		do j = 1, Nvbc
		vbcint_temp = 0.0d0
			do l = 1, j
			vbcint_temp = vbcint_temp+T21quadintktab(i,l)
			enddo
		T21quadavtab(i,j) = vbcint_temp/j
		if(j .eq. 1) T21Temp(i)  = T21quadavtab(i,j) 
		enddo
		!If you want to print out data for Fig. xx in Al-Hamoud et al 2013, uncomment:
    	if (omp%rank .eq.  my_mpi_root) write(*,*) atab(i), T21quadavtab(i, 10)
		!write(*,*) atab(i), T21quadavtab(i, 10)
	enddo 
	
	!renormalize
	do i = 1, Na
		do j = 1, Nvbc
		T21quadavtab(i,j) = T21quadavtab(i,j)-T21Temp(i)
		enddo
	enddo 

	end subroutine compute_21IItot	


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!! 1st order deltas !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	subroutine solve_deltas
	
	  implicit none
	!
	  integer, parameter :: neqn = 12
	!
	  real(mcp) :: abserr
	  !external derivsdeltas
	  integer :: i_step, j, l, int_s
	  integer :: iflag
	  integer :: iwork(5)
	  integer :: n_step
	  real(mcp) :: relerr
	  real(mcp) :: t
	  real(mcp) :: t_out
	  real(mcp) :: t_start
	  real(mcp) :: t_stop
	  real(mcp) :: work(6*neqn+3)
	  real(mcp) :: y(neqn)
	  
#ifdef MPI	  
	  !temp arrays:
	  real(mcp), allocatable :: temp1(:)
	  real(mcp), allocatable :: temp2(:)
	  real(mcp), allocatable :: temp3(:)
	  real(mcp), allocatable :: temp4(:)
	  real(mcp), allocatable :: temp5(:)
	  real(mcp), allocatable :: temp6(:)
	  real(mcp), allocatable :: temp7(:)
	  real(mcp), allocatable :: temp8(:)
	  real(mcp), allocatable :: temp9(:)
	  real(mcp), allocatable :: temp10(:)
	  real(mcp), allocatable :: temp11(:)
	  real(mcp), allocatable :: temp12(:)
	  
	  real(mcp) :: tottemp1(Na*Nvbc*Nk)
	  real(mcp) :: tottemp2(Na*Nvbc*Nk)
	  real(mcp) :: tottemp3(Na*Nvbc*Nk)
	  real(mcp) :: tottemp4(Na*Nvbc*Nk)
	  real(mcp) :: tottemp5(Na*Nvbc*Nk)
	  real(mcp) :: tottemp6(Na*Nvbc*Nk)
	  real(mcp) :: tottemp7(Na*Nvbc*Nk)
	  real(mcp) :: tottemp8(Na*Nvbc*Nk)
	  real(mcp) :: tottemp9(Na*Nvbc*Nk)
	  real(mcp) :: tottemp10(Na*Nvbc*Nk)
	  real(mcp) :: tottemp11(Na*Nvbc*Nk)
	  real(mcp) :: tottemp12(Na*Nvbc*Nk)
	
	  abserr = 0.0000001E+00
	  relerr = 0.0000001E+00
	
	  iflag = 1
	
	  t_start = ainit
	  t_stop = aend
	  t_out = ainit
	  
	  
	  CALL init_mpi_params(Nk)
		
	  do l = 1, omp%size
			recvcounts(l) = recvcounts(l)*Nvbc*Na
			displs(l) = displs(l)*Nvbc*Na
	  enddo
	  
	  allocate(temp1(recvcounts(omp%rank+1)))
	  allocate(temp2(recvcounts(omp%rank+1)))
	  allocate(temp3(recvcounts(omp%rank+1)))
	  allocate(temp4(recvcounts(omp%rank+1)))
	  allocate(temp5(recvcounts(omp%rank+1)))
	  allocate(temp6(recvcounts(omp%rank+1)))
	  allocate(temp7(recvcounts(omp%rank+1)))
	  allocate(temp8(recvcounts(omp%rank+1)))
	  allocate(temp9(recvcounts(omp%rank+1)))
	  allocate(temp10(recvcounts(omp%rank+1)))
	  allocate(temp11(recvcounts(omp%rank+1)))
	  allocate(temp12(recvcounts(omp%rank+1)))
	  

	  int_s  = 1
	  if (omp%rank .eq. my_mpi_root) write(*, '(50a)') "**********************************************100%" 
#else
	  abserr = 0.0000001E+00
	  relerr = 0.0000001E+00
	
	  iflag = 1
	
	  t_start = ainit
	  t_stop = aend
	  t_out = ainit
	  write(*, '(50a)') "**********************************************100%" 
#endif

#ifdef MPI  
	  do l = omp%istart, omp%iend
#else
	  do l = 1, Nk
#endif
	  public_k0 = ktab(l)
	 
	  !initial conditions:
	
	  
	  do j = 1, Nvbc
	  public_vbc = vbctab(j)
	  y(1) = Init%deltac(l)
	  y(2) = 0.000d0
	  y(3) = Init%deltavc(l)
	  y(4) = 0.000d0
	  y(5) = Init%deltab(l)
	  y(6) = 0.000d0
	  y(7) = Init%deltavb(l)
	  y(8) = 0.000d0
	  y(9) = 0.000d0
	  y(10) = 0.000d0
	  y(11) = 0.000d0
	  y(12) = 0.000d0
		do i_step = 1, Na
		tgasPUBLIC = Tgas2tab(i_step)
			if (i_step .eq. 1) then 
#ifdef MPI
			temp1(int_s) = y(1)
			temp2(int_s) = y(2)
			temp3(int_s) = y(3)
			temp4(int_s) = y(4)
			temp5(int_s) = y(5)
			temp6(int_s) = y(6)
			temp7(int_s) = y(7)
			temp8(int_s) = y(8)
			temp9(int_s) = y(9)
			temp10(int_s) = y(10)
			temp11(int_s) = y(11)
			temp12(int_s) = y(12)
#else 
			delta%Recdm(i_step,j,l) = y(1)
			delta%Imcdm(i_step,j,l) = y(2)
			delta%Revcdm(i_step,j,l) = y(3)
			delta%Imvcdm(i_step,j,l) = y(4)
			delta%Rebar(i_step,j,l) = y(5)
			delta%Imbar(i_step,j,l) = y(6)
			delta%Revbar(i_step,j,l) = y(7)
			delta%Imvbar(i_step,j,l) = y(8)
			delta%ReTgas(i_step,j,l) = y(9)
			delta%ImTgas(i_step,j,l) = y(10)
			delta%Rexe(i_step,j,l) = y(11)
			delta%Imxe(i_step,j,l) = y(12)
#endif
			else
			t = atab(i_step-1)
			t_out = atab(i_step)
		
			call rkf45_doublep(derivsdeltas, neqn, y, t, t_out, relerr, abserr, iflag, work, iwork )
#ifdef MPI
			temp1(int_s) = y(1)
			temp2(int_s) = y(2)
			temp3(int_s) = y(3)
			temp4(int_s) = y(4)
			temp5(int_s) = y(5)
			temp6(int_s) = y(6)
			temp7(int_s) = y(7)
			temp8(int_s) = y(8)
			temp9(int_s) = y(9)
			temp10(int_s) = y(10)
			temp11(int_s) = y(11)
			temp12(int_s) = y(12)
#else			
			delta%Recdm(i_step,j,l) = y(1)
			delta%Imcdm(i_step,j,l) = y(2)
			delta%Revcdm(i_step,j,l) = y(3)
			delta%Imvcdm(i_step,j,l) = y(4)
			delta%Rebar(i_step,j,l) = y(5)
			delta%Imbar(i_step,j,l) = y(6)
			delta%Revbar(i_step,j,l) = y(7)
			delta%Imvbar(i_step,j,l) = y(8)
			delta%ReTgas(i_step,j,l) = y(9)
			delta%ImTgas(i_step,j,l) = y(10)
			delta%Rexe(i_step,j,l) = y(11)
			delta%Imxe(i_step,j,l) = y(12)	
			!write(*,*) delta%Recdm(i_step,j,l)
#endif
			endif
			int_s = int_s+1
			enddo 
		enddo
		
	  write(*,'(a)',advance='no')  '*'
	  
	  end do
#ifdef MPI	  
	  
		  !making sure all proc. share the computed table:             
	  call MPI_ALLGATHERV(temp1, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp1, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp2, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp2, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror)                     
	  call MPI_ALLGATHERV(temp3, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp3, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp4, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp4, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp5, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp5, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp6, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp6, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp7, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp7, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp8, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp8, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp9, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp9, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp10, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp10, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp11, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp11, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp12, omp%steps*Na*Nvbc, MPI_REAL8, &
						   tottemp12, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror)                       
	   int_s = 1
	   do l = 1, Nk
			do j = 1, Nvbc
				do i_step = 1, Na
					delta%Recdm(i_step,j,l) = tottemp1(int_s)
					delta%Imcdm(i_step,j,l) = tottemp2(int_s)
					delta%Revcdm(i_step,j,l) = tottemp3(int_s)
					delta%Imvcdm(i_step,j,l) = tottemp4(int_s)
					delta%Rebar(i_step,j,l) = tottemp5(int_s)
					delta%Imbar(i_step,j,l) = tottemp6(int_s)
					delta%Revbar(i_step,j,l) = tottemp7(int_s)
					delta%Imvbar(i_step,j,l) = tottemp8(int_s)
					delta%ReTgas(i_step,j,l) = tottemp9(int_s)
					delta%ImTgas(i_step,j,l) = tottemp10(int_s)
					delta%Rexe(i_step,j,l) = tottemp11(int_s)
					delta%Imxe(i_step,j,l) = tottemp12(int_s)
					int_s = int_s + 1
				enddo 
			enddo 
	   enddo 

	  deallocate(displs)
	  deallocate(recvcounts)
	  
	  deallocate(temp1)
	  deallocate(temp2)
	  deallocate(temp3)
	  deallocate(temp4)
	  deallocate(temp5)
	  deallocate(temp6)
	  deallocate(temp7)
	  deallocate(temp8)
	  deallocate(temp9)
	  deallocate(temp10)
	  deallocate(temp11)
	  deallocate(temp12)
	 
	  if (omp%rank .eq. 0) write(*,*) ""
#else
	  write(*,*) ""
#endif
	  
	  
	  return
	end subroutine solve_deltas
	
	
	subroutine solve_dedT
	
	  implicit none
	!
	  integer, parameter :: neqn = 2
	!
	  real(mcp) :: abserr
	  !external derivs
	  integer :: i_step, j, int_s
	  integer :: iflag
	  integer :: iwork(5)
	  integer :: n_step
	  real(mcp) :: relerr
	  real(mcp) :: t
	  real(mcp) :: t_out
	  real(mcp) :: t_start
	  real(mcp) :: t_stop
	  real(mcp) :: work(6*neqn+3)
	  real(mcp) :: y(neqn)
	  

#ifdef MPI	  
	  !temp arrays:
	  real(mcp), allocatable :: temp1(:)
	  real(mcp), allocatable :: temp2(:)
	  
	  real(mcp) :: tottemp1(Na*Nvbc)
	  real(mcp) :: tottemp2(Na*Nvbc)
	!
	
	  CALL init_mpi_params(Nvbc)
		
	  do j = 1, omp%size
			recvcounts(j) = recvcounts(j)*Na
			displs(j) = displs(j)*Na
	  enddo
	  allocate(temp1(recvcounts(omp%rank+1)))
	  allocate(temp2(recvcounts(omp%rank+1)))
	  
	  abserr = 0.000000000000001E+00
	  relerr = 0.00000000001E+00
	
	  iflag = 1
	
	  t_start = ainit
	  t_stop = aend
	
	  !n_step = 99
	
	  t_out = ainit
	  !initial conditions:
	
	  int_s = 1
	  if (omp%rank .eq. my_mpi_root) write(*, '(45a)') "*****************************************100%" 
#else
	  abserr = 0.000000000000001E+00
	  relerr = 0.00000000001E+00
	
	  iflag = 1
	
	  t_start = ainit
	  t_stop = aend
	
	  t_out = ainit
	  
	  write(*, '(45a)') "*****************************************100%"
#endif

#ifdef MPI
	  do j = omp%istart, omp%iend
#else
	  do j = 1, Nvbc
#endif
	  public_vbc = vbctab(j)
	  
	  y(1) = 0.0d0
	  y(2) = 0.0d0
		do i_step = 1, Na
		tgasPUBLIC = Tgas2tab(i_step)
		if (i_step .eq. 1) then 
#ifdef MPI
			temp1(int_s) = y(1)
			temp2(int_s) = y(2)
#else
			de2tab(i_step, j) = y(1)
			dT2tab(i_step, j) = y(2)
#endif
		else
		t = atab(i_step-1)
		t_out = atab(i_step)
	
		call rkf45_doublep(derivs, neqn, y, t, t_out, relerr, abserr, iflag, work, iwork )
#ifdef MPI
		temp1(int_s) = y(1)
		temp2(int_s) = y(2)
#else
		de2tab(i_step, j) = y(1)
		dT2tab(i_step, j) = y(2)
#endif
		endif
		
		int_s = int_s+1
		enddo 
		write(*,'(a)',advance='no')  '*'
	  end do
#ifdef MPI	  
	  call MPI_ALLGATHERV(temp1, omp%steps*Na, MPI_REAL8, &
						   tottemp1, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  call MPI_ALLGATHERV(temp2, omp%steps*Na, MPI_REAL8, &
						   tottemp2, recvcounts, displs, MPI_Real8, &
						  MPI_COMM_WORLD, omp%ierror) 
	  int_s = 1
	  do j = 1, Nvbc
		  do i_step = 1, Na
		  
		  de2tab(i_step, j) = tottemp1(int_s)
		  dT2tab(i_step, j) = tottemp2(int_s)
		  int_s = int_s+1
		  enddo 
	  enddo
	  
	  !renormalize
	  do i_step = 1, Na	
	  	do j = 1, Nvbc		  
		  
		  Nde2tab(i_step, j) = de2tab(i_step, j) - de2tab(i_step, 1)
		  NdT2tab(i_step, j) = dT2tab(i_step, j) - dT2tab(i_step, 1)
		 
		  enddo 
	  enddo 
	  
	  deallocate(displs)
	  deallocate(recvcounts)
	  
	  deallocate(temp1)
	  deallocate(temp2)
	  
	  if (omp%rank .eq. my_mpi_root) write(*,*) ""
#else
	  !renormalize
	  do i_step = 1, Na	
	  	do j = 1, Nvbc		  
		  
		  Nde2tab(i_step, j) = de2tab(i_step, j) - de2tab(i_step, 1)
		  NdT2tab(i_step, j) = dT2tab(i_step, j) - dT2tab(i_step, 1)
		  write(*,*) Nde2tab(i_step, j)
		  enddo 
	  enddo 
	 
	 write(*,*) ""
#endif
	  	  
	 return
	end subroutine solve_dedT
	
	subroutine derivs( a, y, yp )
	
	  implicit none
	
	  real(mcp) :: a
	  real(mcp) :: y(2)
	  real(mcp) :: yp(2)
	  real(mcp) :: vbc, Ha, Tg, nhb
	
	  
	  vbc = public_vbc
	  Ha = Hubble(a)
	  nhb = nhbar(a)
	  if (Compute_Tgas) then
	  Tg = tgasPUBLIC
	  else
	  Tg = Tgasbar(a)
	  endif
	  if(RCBparams%DMAN_on) then
	  yp(1) = DxebarDa(a)/xebar(a)*(y(1) + dlnAdlnTm(a)*y(2)) + sourcede2(a,vbc)/(a*Ha) - &
	  			Urate(a)*y(1)/(3.0d0*nhb)
	  yp(2) = (Gcompton(a)/(a*Ha))*((Tcmb(a)/Tg-1.0d0)*(y(1)-y(2))-y(2)) + & 
	  		  Gchi(a)/(a*Ha)*(-y(2))/Tg + sourcedT2(a,vbc)/(a*Ha)
				 		  
	  else
	  yp(1) = DxebarDa(a)/xebar(a)*(y(1) + dlnAdlnTm(a)*y(2)) + sourcede2(a,vbc)/(a*Ha)
	  yp(2) = (Gcompton(a)/(a*Ha))*((Tcmb(a)/Tg-1.0d0)*(y(1)-y(2))-y(2)) + & 
				 sourcedT2(a,vbc)/(a*Ha)			   
	  endif
	  return
	end subroutine derivs
	
	
	subroutine derivsdeltas( a, y, yp )
	
	  implicit none
	!
	  real(mcp) :: a
	  real(mcp) :: y(12)
	  real(mcp) :: yp(12)
	  real(mcp) :: vbc
	  real(mcp) :: k0
	  ! to speed up code and call functions only once
	  real(mcp) :: Ha
	  real(mcp) :: cs2
	  real(mcp) :: Tg, Tc, fac1, fac2, rat, fac3, fac4, fac5, RateDA, rat2
	  
	  Ha = Hubble(a)
	  cs2 = soundspeed2(a)
	  if (Compute_Tgas) then
	  Tg = tgasPUBLIC
	  else
	  Tg = Tgasbar(a)
	  endif
	  Tc = Tcmb(a)
	  fac1 = Gcompton(a)
	  fac2 = fast_dlnCdlnR(a)
	  rat = DxebarDa(a)/xebar(a)
	  fac3 = dlnAdlnTm(a)
	  fac4 = Gchi(a)
	  fac5 =  2.0d0*xebar(a)/(1+2.0d0*xebar(a))
	  rat2 = 1.d0/(nhbar(a))
	  RateDA = Urate(a)
	  !conversion:
	  !y(1) = Re[deltac], y(2) = Im[deltac]
	  !y(3) = Re[deltcavc], y(4) = Im[deltavc]
	  !y(5) = Re[deltab], y(6) = Im[deltab]
	  !y(7) = Re[deltavb], y(8) = Im[deltavb]
	  !y(9) = Re[deltaT], y(10) = Im[deltaT]
	  !y(11) = Re[deltae], y(12)  = Im[deltae]
	  
	  !public parameters for looping
	  
	  vbc = public_vbc
	  k0 = public_k0
	  
	  

	  yp(1) = (1.0d0/(a*Ha))*(-(k0*vbc*ainit/a**(2.0d0))*y(2)-y(3))
	  yp(2) = (1.0d0/(a*Ha))*((k0*vbc*ainit/a**(2.0d0))*y(1)-y(4))
	  
	  yp(3) = -(k0*vbc*ainit/(a**3*Ha)*y(4))-1.5d0*(derCMBparams%Hub0**2)* &
			   (CMBparams%omegab0*y(5)+CMBparams%omegac0*y(1))/(a**4*Ha) -2.0d0*y(3)/a
	  yp(4) = (k0*vbc*ainit/(a**3*Ha)*y(3))-1.5d0*(derCMBparams%Hub0**2)* &
			   (CMBparams%omegab0*y(6)+CMBparams%omegac0*y(2))/(a**4*Hubble(a)) -2.0d0*y(4)/a
			   
	  yp(5) = -y(7)/(a*Ha) 
	  yp(6) = -y(8)/(a*Ha) 
	  	  
	  yp(7) = -1.5d0*(derCMBparams%Hub0**2)*(CMBparams%omegab0*y(5)+CMBparams%omegac0*y(1))/(a**4*Ha) - &
				2.d0*y(7)/a + cs2*k0**2*(y(5)+y(9))/(a**3*Ha)
	  yp(8) = -1.5d0*(derCMBparams%Hub0**2)*(CMBparams%omegab0*y(6)+CMBparams%omegac0*y(2))/(a**4*Ha) - &
				2.d0*y(8)/a + cs2*k0**2*(y(6)+y(10))/(a**3*Ha)	
				
	  !dark matter anihilation on?		
	  if(RCBparams%DMAN_on) then 
	  yp(9) = 2.0d0*yp(5)/3.0d0 + fac1*((Tc/Tg-1.0d0)*(y(11)-y(9))-y(9))/(a*Ha) + &
	  			fac4/Tg*(-y(9)-y(5)+2.d0*y(1)+fac5*y(11))/(a*Ha)
	  yp(10) = 2.0d0*yp(6)/3.0d0 + fac1*((Tc/Tg-1.0d0)*(y(12)-y(10))-y(10))/(a*Ha) +&
	  			fac4/Tg*(-y(10)-y(6)+2.d0*y(2)+fac5*y(12))/(a*Ha)
	  !C  = 1, have to check this more carefully later
	  yp(11) = rat*(y(11)+y(5)+fac2*(y(7)/(3.d0*Ha)-y(5))+fac3*y(9)) + &
	  			rat2*RateDA*(-y(11)-y(5)/xebar(a)+y(5))/3.0d0
	  yp(12) = rat*(y(12)+y(6)+fac2*(y(8)/(3.d0*Ha)-y(6))+fac3*y(10)) + &
	  			rat2*RateDA*(-y(12)-y(6)/xebar(a)+y(6))/3.0d0
	  else
	  yp(9) = 2.0d0*yp(5)/3.0d0 + fac1*((Tc/Tg-1.0d0)*(y(11)-y(9))-y(9))/(a*Ha)
	  yp(10) = 2.0d0*yp(6)/3.0d0 + fac1*((Tc/Tg-1.0d0)*(y(12)-y(10))-y(10))/(a*Ha)
	  
	  yp(11) = rat*(y(11)+y(5)+fac2*(y(7)/(3.d0*Ha)-y(5))+fac3*y(9))
	  yp(12) = rat*(y(12)+y(6)+fac2*(y(8)/(3.d0*Ha)-y(6))+fac3*y(10))
	  
	  endif
	  
	  return
	end subroutine derivsdeltas    
	
end module perturbations
