!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!module to read in several files. Can read precomuted deltas. Mostly for electron density!
!perturbations computed through Yacine Al-Hamound's code. Currenlty also reads transfer!!
!functions, which are needed for setting the initial conditions. In future, coupling this!
!to CAMB sources, this should no longer be necessary!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!D. Meerburg 2014!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module readfiles

use settings 
use mpi_params
use background 


implicit none

real(mcp) :: xetab(xezmax,3)
real(mcp) :: aitab(aimax,2)
!real(mcp) :: transferz1000(Ntrans,3)
!real(mcp) :: transferz1020(Ntrans,3)

public xetab, aitab
!public transferz1000, transferz1020

contains

	subroutine read_files(deltas)

		integer ierror
		integer ::  error,j
		character(4)  :: alt1 = '.txt', alt2 = '_', alt3 = '/',alt4 = '.dat'
		character(80) :: del1, del2, del3, del4, del5
		character(80) :: data_dir, delc, delb, delT, delvb, deles
		Type(fluctuations) :: deltas
#ifdef MPI		
		type(mpi_pars) :: omp

	
	
	    call MPI_COMM_RANK(MPI_COMM_WORLD, omp%rank, omp%ierror)

		!now read in files:
		data_dir = 'deltas/'
		if (feedback > 0 .and. omp%rank .eq. 0) then
			write(*,*) "Reading delta files a, vbc, k, Re[del], Im[del]"
		endif
#endif		
		del1 = 'deltac'
		del2 = 'deltab'
		del3 = 'deltaes'
		del4 = 'deltaT'
		del5 = 'deltavb'
		
		delc = trim(data_dir)//trim(del1)//trim(alt4)
		delb = trim(data_dir)//trim(del2)//trim(alt4)
		deles = trim(data_dir)//trim(del3)//trim(alt4)
		delT = trim(data_dir)//trim(del4)//trim(alt4)
		delvb = trim(data_dir)//trim(del5)//trim(alt4)
		open(unit=21,file=delc,form='formatted',status='old')
		
		!write(*,*) "reading files"    
		do j = 1, zkvmax
			read(21,*) deltas%cdm(j,1:5)
		enddo
		close(21)
		
	    open(unit=21,file=delb,form='formatted',status='old')
		
		!write(*,*) "reading files"    
		do j = 1, zkvmax
			read(21,*) deltas%bar(j,1:5)
		enddo
		close(21)
		
	    open(unit=21,file=deles,form='formatted',status='old')
		
		!write(*,*) "reading files"    
		do j = 1, zkvmax
			read(21,*) deltas%xe(j,1:5)
		enddo
		close(21)
		
	    open(unit=21,file=delT,form='formatted',status='old')
		
		!write(*,*) "reading files"    
		do j = 1, zkvmax
			read(21,*) deltas%Tgas(j,1:5)
		enddo
		close(21)
		
	    open(unit=21,file=delvb,form='formatted',status='old')
		
		!write(*,*) "reading files"    
		do j = 1, zkvmax
			read(21,*) deltas%vbar(j,1:5)
		enddo
		close(21)
#ifdef MPI		
		if (feedback > 0 .and. omp%rank .eq. 1) then 
		write(*,*) "Files have been read"
		endif
#endif		
	end subroutine read_files
	
	subroutine read_planckxe
		
		integer ierror
		integer ::  error, j
		character(4)  :: alt1 = '.txt', alt2 = '_', alt3 = '/',alt4 = '.dat'
		character(80) :: del1, del2, del3, del4, del5
		character(80) :: data_dir, filename, ps	
#ifdef MPI		

		type(mpi_pars) :: omp
		
		call MPI_COMM_RANK(MPI_COMM_WORLD, omp%rank, omp%ierror)
		
		if (feedback > 1 .and. omp%rank .eq. 0) then
			write(*,*) "Reading Planck Xe file"
		endif
#endif
        data_dir = 'data/'
        
        filename = 'xePlanck'
        ps = trim(data_dir)//trim(filename)//trim(alt4)
        
        
        open(unit=21,file=ps,form='formatted',status='old')
		
		!write(*,*) "reading files"    
		do j = 1, xezmax
			read(21,*) xetab(j,1:3)
		enddo
		close(21)
		
	end subroutine read_planckxe


    subroutine read_alphainf
		
		integer ierror
		integer ::  error, j
		character(4)  :: alt1 = '.txt', alt2 = '_', alt3 = '/',alt4 = '.dat'
		character(80) :: del1, del2, del3, del4, del5
		character(80) :: data_dir, filename, ps
#ifdef MPI				
		type(mpi_pars) :: omp
		
		call MPI_COMM_RANK(MPI_COMM_WORLD, omp%rank, omp%ierror)
		
		
		if (feedback > 0 .and. omp%rank .eq. 0) then
			write(*,*) "Reading Alpha_inf file"
		endif
#endif 
        data_dir = 'data/'
       
        filename = 'Alpha_inf'
        ps = trim(data_dir)//trim(filename)//trim(alt4)
        
        
        open(unit=21,file=ps,form='formatted',status='old')
		
		!write(*,*) "reading files"    
		do j = 1, aimax
			read(21,*) aitab(j,1:2)
		enddo
		close(21)
		
	end subroutine read_alphainf
	
!comment this in if you (want to) use precomputed tranfer functions (have to recompute if you change 
!cosmology, but not, for example, if you run different primordial model)

!	subroutine read_transfer
!		
!		integer ierror
!		integer ::  error, j
!		character(4)  :: alt1 = '.txt', alt2 = '_', alt3 = '/',alt4 = '.dat'
!		character(80) :: data_dir, filename, ps
!		type(mpi_pars) :: omp
!		
!		call MPI_COMM_RANK(MPI_COMM_WORLD, omp%rank, omp%ierror)
!		
!		
!		if (feedback > 0 .and. omp%rank .eq. 0) then
!			write(*,*) "Reading transfer functions"
!		endif
!        data_dir = 'data/'
!        
!        filename = 'mys_transfer_outz1000'
!        ps = trim(data_dir)//trim(filename)//trim(alt4)
!        
!        
!        open(unit=21,file=ps,form='formatted',status='old')
!		
!		!write(*,*) "reading files"    
!		do j = 1, Ntrans
!			read(21,*) transferz1000(j,1:3)
!			transferz1000(j,1) = transferz1000(j,1)*CMBparams%h0
!		enddo
!		close(21)
!		
!		filename = 'mys_transfer_outz1020'
!        ps = trim(data_dir)//trim(filename)//trim(alt4)
!        
!        
!        open(unit=21,file=ps,form='formatted',status='old')
!		
!		!write(*,*) "reading files"    
!		do j = 1, Ntrans
!			read(21,*) transferz1020(j,1:3)
!			transferz1020(j,1) = transferz1020(j,1)*CMBparams%h0
!		enddo
!		close(21)
!		
!	end subroutine read_transfer
	
	

end module readfiles